unit vars;

interface

uses
  Classes, Dialogs, Windows, Graphics, ExtCtrls;

const
  Filter: array [0..5] of string = (
   'Zone files|*.zon|All files|*.*',
   'Room files|*.wld|All files|*.*',
   'Object files|*.obj|All files|*.*',
   'Mob files|*.mob|All files|*.*',
   'Shop files|*.shp|All files|*.*',
   'Trigger files|*.trg|All files|*.*'
  );

  DefaultExt: array [0..5] of string = (
   'zon', 'wld', 'obj', 'mob', 'shp', 'trg'
  );

type
  TDice = record { Dice roll: nDd+a }
    n: byte;
    d: byte;
    a: smallint;
  end;

  TSkill = record
    Number: word;  { }
    Percent: smallint; {, %}
  end;

  TSpell = record
    Number: word;  { }
    Qty: smallint; { }
  end;

  TExtraDesc = class
    Alias: string;
    Desc: TStringList;
    constructor Create;
    destructor Destroy; override;
  end;

  TExtraDescList = class (TList)
    destructor Destroy; override;
  end;

  TAddParam = record
    Number: word;
    Value: smallint;
  end;

  TObj = class
    Number: integer;          {    }
    Alias: string;
    Imen, Rod, Dat, Vin, Tvor, Pred: string;
    Desc: string;
    Action: string;
    TrenSkill: smallint;
    MagicVector: string;
    MaxDurab, CurrDurab: smallint;
    Material: word;
    Pol: byte;
    Timer: smallint;
    Spell: smallint;
    SpellLevel: byte;
    Effects, Discomfort, Bans: string;
    Kind: byte;
    Flags, WearFlags: string;
    Param1: string;
    Param2, Param3, Param4: integer;
    Weight: byte;
    Price, RentInv, RentWear: integer;
    Extra: TExtraDescList;
    AddParam: TList;
    Triggers: TList;
    constructor Create(_number: integer);
    destructor Destroy; override;
  end;

  TObjList = class (TList)
    procedure Add(_number: integer);
    function Convert: integer;
    procedure SaveToFile(FileName: string);
    destructor Destroy; override;
  end;

  TMob = class
    Number: integer;          {    }
    Alias: string;
    Imen, Rod, Dat, Vin, Tvor, Pred: string;
    Desc: string;
    Look: TStringList;
    Flags: string;
    Affects: string;
    Align: integer;
    Level: word;
    hitroll: shortint;
    AC: shortint;
    Hits: TDice;
    Damage: TDice;
    Money: TDice;
    Exp: integer;
    PosLoad, PosDefault: byte;
    Pol: byte;
    {additions}
    BareHandAttack: shortint;
    Destination: integer;
    Str, Int, Wis, Dex, Con, Cha: byte;
    LikeWork: integer;
    MaxFactor: integer;
    ExtraAttack: word;
    Model: byte;
    Size: integer;
    Height: integer;
    Weight: integer;
    Special: string;
    Spells: TList;
    Skills: TList;
    Helper: TList;
    Triggers: TList;
    constructor Create(_number: integer);
    function FindSpellNum(SpellNum: word): integer;
    destructor Destroy; override;
  end;

  TMobList = class (TList)
    procedure Add(_number: integer);
    function Convert: integer;
    procedure SaveToFile(FileName: string);
    destructor Destroy; override;
  end;

  TGoods = record
    Kind: word;  { }
    Qty: smallint; {  }
  end;

  TShop = class
    Number: integer;               {  }
    Goods: TList;                  { ,    }
    CoeffSell: real;               {   }
    CoeffBuy: real;                {   }
    CoeffChange: real;             {   }
    GoodsBuy: TStringList;         {   }
    GoodsChange: TStringList;      {   }
    Msg1: string;                  {     (%s   )}
    Msg2: string;                  {    }
    Msg3: string;                  {   }
    Msg4: string;                  {      }
    Msg5: string;                  {      }
    Msg6: string;                  {   (%d   )}
    Msg7: string;                  {    (%d   )}
    Emotion: integer;              {    }
    BitVector: string;             {}
    Seller: integer;               { ( )}
    NotSellVector: string;         {     }
    Rooms: TList;                  {   }
    Open1: byte;                   {  1}
    Close1: byte;                  {  1}
    Open2: byte;                   {  2}
    Close2: byte;                  {  2}
    constructor Create(_number: integer);
    destructor Destroy; override;
  end;

  TShopList = class (TList)
    procedure Add(_number: integer);
    function Convert: integer;
    procedure SaveToFile(FileName: string);
    destructor Destroy; override;
  end;

  TExit = record
    Name: string;
    Desc: string;
    Flag: byte;
    Key: integer;
    ExitRoom: integer;
    DefaultValue: byte;
  end;

  TArrayExits = array [0..5] of TExit;

  TRoom = class
    Number: integer;
    Name: string;
    Desc: TStringList;
    Flags: string;
    Sector: byte;
    Exits: TArrayExits;
    Extra: TExtraDescList;
    Triggers: TList;
    constructor Create(_number: integer);
    destructor Destroy; override;
  end;

  TRoomList = class (TList)
    procedure Add(_number: integer);
    function Convert: integer;
    procedure SaveToFile(FileName: string);
    destructor Destroy; override;
  end;

  TZone = class
    Number: integer;
    Name: string;
    RepopTime: integer;
    RepopType: integer;
    Items: TStringList;
    constructor Create;
    procedure Clear;
    function Convert: integer;
    procedure SaveToFile(FileName: string);
    destructor Destroy; override;
  end;

var
  Zone: TZone;
  RoomList: TRoomList;
  ObjList: TObjList;
  MobList: TMobList;
  ShopList: TShopList;
  StrCounter: integer;

implementation

uses
  main, SysUtils, StrUtils;

{ Internal }

function PrepareFlags(s: string): string;
var
  i: integer;
  tStr: string;
begin
  tStr := '';
  for i := 1 to Length(s) do
   tStr := tStr + s[i] + '0';
  Result := tStr;
end;

function GetNextStr: string;
begin
  Inc(StrCounter);
  Result := Form1.SourceMemo.Lines[StrCounter];
end;

function EndOfLines: boolean;
begin
  Result := StrCounter >= Form1.SourceMemo.Lines.Count;
end;

{ Dice}

procedure StrToDice(s: string; var Dice: TDice);
begin
  {NdD+A}
  Dice.n := StrToInt(ExtractWord(1, s, ['d']));
  Delete(s, 1, Pos('d', s));
  Dice.d := StrToInt(ExtractWord(1, s, ['+']));
  Dice.a := StrToInt(ExtractWord(2, s, ['+']));
end;

procedure DiceToStr(var s: string; Dice: TDice);
begin
  s := IntToStr(Dice.n) + 'd' + IntToStr(Dice.d) + '+' + IntToStr(Dice.a);
end;

{ TExtraDesc }

constructor TExtraDesc.Create;
begin
  Alias := '';
  Desc := TStringList.Create;
  inherited Create;
end;

destructor TExtraDesc.Destroy;
begin
  Desc.Free;
  inherited Destroy;
end;

{ TExtraDescList }

destructor TExtraDescList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TExtraDesc(Items[i]).Free;
  inherited Destroy;
end;

{ TObj }

constructor TObj.Create;
begin
  Number := _number;
  Alias := '';
  Imen := '';
  Rod := ''; Dat := ''; Vin := ''; Tvor := ''; Pred := '';
  Desc := '';
  Action := '';
  TrenSkill := 0;
  MagicVector := '';
  MaxDurab := 0; CurrDurab := 0;
  Material := 0;
  Pol := 0;
  Timer := 0;
  Spell := 0;
  SpellLevel := 0;
  Effects := ''; Discomfort := ''; Bans := '';
  Kind := 0;
  Flags := ''; WearFlags := '';
  Param1 := '';
  Param2 := 0; Param3 := 0; Param4 := 0;
  Weight := 0;
  Price := 0; RentInv := 0; RentWear := 0;
  Extra := TExtraDescList.Create;
  AddParam := TList.Create;
  Triggers := TList.Create;
  inherited Create;
end;

destructor TObj.Destroy;
begin
  Extra.Free;
  AddParam.Free;
  Triggers.Free;
  inherited Destroy;
end;

{ TObjList }

procedure TObjList.Add(_number: integer);
var
  NewObj: TObj;
begin
  NewObj := TObj.Create(_number);
  inherited Add(NewObj);
end;

function TObjList.Convert: integer;
var
  tmpStr: string;
  i: integer;
  AddPar: TAddParam;
  ExtraDesc: TExtraDesc;
begin
  StrCounter := -1;
  try
    Clear;
    repeat
      tmpStr := GetNextStr;
    until EndOfLines or ((tmpStr <> '') and (tmpStr[1] = '#'));
    if not EndOfLines then
    repeat
      System.Delete(tmpStr, 1, 1);
      Add(StrToInt(tmpStr));
      with TObj(Items[Count-1]) do
      begin
        tmpStr := GetNextStr; Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Imen := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Rod := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Dat := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Vin := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Tvor := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Pred := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Desc := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Action := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr;
        MagicVector := ExtractWord(1, tmpStr, [' ']);
        Val(MagicVector, TrenSkill, i);
        if i <> 0 then TrenSkill := 0;
        if MagicVector = '0' then MagicVector := '';
        MaxDurab := StrToInt(ExtractWord(2, tmpStr, [' ']));
        CurrDurab := StrToInt(ExtractWord(3, tmpStr, [' ']));
        Material := StrToInt(ExtractWord(4, tmpStr, [' ']));
        tmpStr := GetNextStr;
        Pol := StrToInt(ExtractWord(1, tmpStr, [' ']));
        Timer := StrToInt(ExtractWord(2, tmpStr, [' ']));
        Spell := StrToInt(ExtractWord(3, tmpStr, [' ']));
        SpellLevel := StrToInt(ExtractWord(4, tmpStr, [' ']));
        tmpStr := GetNextStr;
        Effects := ExtractWord(1, tmpStr, [' ']);
        if Effects = '0' then Effects := '';
        Effects := PrepareFlags(Effects);
        Discomfort := ExtractWord(2, tmpStr, [' ']);
        if Discomfort = '0' then Discomfort := '';
        Discomfort := PrepareFlags(Discomfort);
        Bans := ExtractWord(3, tmpStr, [' ']);
        if Bans = '0' then Bans := '';
        Bans := PrepareFlags(Bans);
        tmpStr := GetNextStr;
        Kind := StrToInt(ExtractWord(1, tmpStr, [' ']));
        Flags := ExtractWord(2, tmpStr, [' ']);
        if Flags = '0' then Flags := '';
        Flags := PrepareFlags(Flags);
        WearFlags := ExtractWord(3, tmpStr, [' ']);
        if WearFlags = '0' then WearFlags := '';
        WearFlags := PrepareFlags(WearFlags);
        tmpStr := GetNextStr;
        Param1 := ExtractWord(1, tmpStr, [' ']);
        Param2 := StrToInt(ExtractWord(2, tmpStr, [' ']));
        Param3 := StrToInt(ExtractWord(3, tmpStr, [' ']));
        Param4 := StrToInt(ExtractWord(4, tmpStr, [' ']));
        tmpStr := GetNextStr;
        Weight := StrToInt(ExtractWord(1, tmpStr, [' ']));
        Price := StrToInt(ExtractWord(2, tmpStr, [' ']));
        RentInv := StrToInt(ExtractWord(3, tmpStr, [' ']));
        RentWear := StrToInt(ExtractWord(4, tmpStr, [' ']));
        tmpStr := GetNextStr;
        while (tmpStr<>'') and (tmpStr[1] = 'E') do
        begin
          ExtraDesc := TExtraDesc.Create;
          tmpStr := GetNextStr;
          ExtraDesc.Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
          tmpStr := GetNextStr;
          while tmpStr[Length(tmpStr)] <> '~' do
          begin
            ExtraDesc.Desc.Add(tmpStr);
            tmpStr := GetNextStr;
          end;
          tmpStr := Copy(tmpStr, 1, Length(tmpStr)-1);
          if tmpStr <> '' then
           ExtraDesc.Desc.Add(tmpStr);
          Extra.Add(ExtraDesc);
          tmpStr := GetNextStr;
        end;
        while (tmpStr<>'') and (tmpStr[1] = 'A') do
        begin
          tmpStr := GetNextStr;
          AddPar.Number := StrToInt(ExtractWord(1, tmpStr, [' ']));
          AddPar.Value := StrToInt(ExtractWord(2, tmpStr, [' ']));
          AddParam.Add(TObject(AddPar));
          tmpStr := GetNextStr;
        end;
        while (tmpStr<>'') and (tmpStr[1] = 'T') do
        begin
          Triggers.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
          tmpStr := GetNextStr;
        end;
      end;  
      {...}
      while (not EndOfLines) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
       tmpStr := GetNextStr;
    until EndOfLines;
    Result := 0;
  except
    MessageDlg('     ' + IntToStr(StrCounter), mtError, [mbOk], 0);
    Result := StrCounter;
  end;
end;

procedure TObjList.SaveToFile(FileName: string);
var
  f: TextFile;
  i, j: integer;
  s1, s2: string;
begin
  try
    AssignFile(f, Filename); Rewrite(f);
    try
      WriteLn(f, '* BRusMUD object file v1.0');
      for i := 0 to Count-1 do
       with TObj(Items[i]) do
      begin
        WriteLn(f, Format('#%d', [Number]));
        WriteLn(f, Alias + '~');
        WriteLn(f, Imen + '~');
        WriteLn(f, Rod + '~');
        WriteLn(f, Dat + '~');
        WriteLn(f, Vin + '~');
        WriteLn(f, Tvor + '~');
        WriteLn(f, Pred + '~');
        WriteLn(f, Desc + '~');
        WriteLn(f, Action + '~');
        if Kind = 25 then
        begin
          if MagicVector = '' then s1 := '0'
          else s1 := MagicVector;
        end else
         s1 := IntToStr(TrenSkill);
        WriteLn(f, Format('%s %d %d %d', [s1, MaxDurab, CurrDurab, Material]));
        WriteLn(f, Format('%d %d %d %d', [Pol, Timer, Spell, SpellLevel]));
        if Effects = '' then Write(f, '0 ')
        else Write(f, Effects + ' ');
        if Discomfort = '' then Write(f, '0 ')
        else Write(f, Discomfort + ' ');
        if Bans = '' then WriteLn(f, '0')
        else WriteLn(f, Bans);
        if Flags = '' then s1 := '0'
        else s1 := Flags;
        if WearFlags = '' then s2 := '0'
        else s2 := WearFlags;
        WriteLn(f, Format('%d %s %s', [Kind, s1, s2]));
        if Param1 = '' then s1 := '0'
        else s1 := Param1;
        WriteLn(f, Format('%s %d %d %d', [s1, Param2, Param3, Param4]));
        WriteLn(f, Format('%d %d %d %d', [Weight, Price, RentInv, RentWear]));
        for j := 0 to Extra.Count-1 do
         with TExtraDesc(Extra.Items[j]) do
        begin
          WriteLn(f, 'E');
          WriteLn(f, Alias + '~');
          WriteLn(f, Desc.Text + '~');
        end;
        for j := 0 to AddParam.Count-1 do
         with TAddParam(AddParam.Items[j]) do
        begin
          WriteLn(f, 'A');
          WriteLn(f, Format('%d %d', [Number, Value]));
        end;
        for j := 0 to Triggers.Count-1 do
         WriteLn(f, Format('T %d', [integer(Triggers.Items[j])]));
      end;
      WriteLn(f, '$'#13#10'$');
    finally
      CloseFile(f);
    end;
  except
    MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TObjList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TObj(Items[i]).Free;
  inherited Destroy;
end;

{ TMob }

constructor TMob.Create;
begin
  Number := _number;
  Alias := '';
  Imen := '';
  Rod := ''; Dat := ''; Vin := ''; Tvor := ''; Pred := '';
  Desc := '';
  Look := TStringList.Create;
  Flags := 'h0';
  Affects := '';
  Align := 0;
  Level := 1;
  hitroll := 0;
  AC := 0;
  Hits.n := 0; Hits.d := 0; Hits.a := 0;
  Damage.n := 0; Damage.d := 0; Damage.a := 0;
  Money.n := 0; Money.d := 0; Money.a := 0;
  Exp := 0;
  PosLoad := 8; PosDefault := 8;
  Pol := 1;
  BareHandAttack := 0;
  Destination := 0;
  Str := 11; Int := 11; Wis := 11; Dex := 11; Con := 11; Cha := 11;
  LikeWork := 0;
  MaxFactor := 0;
  ExtraAttack := 0;
  Model := 0;
  Size := 0;
  Height := 0;
  Weight := 0;
  Special := '';
  Spells := TList.Create;
  Skills := TList.Create;
  Helper := TList.Create;
  Triggers := TList.Create;
  inherited Create;
end;

function TMob.FindSpellNum(SpellNum: word): integer;
var
  i, Res: integer;
begin
  Res := -1;
  for i := 0 to Spells.Count-1 do
   if TSpell(Spells.Items[i]).Number = SpellNum then
  begin
    Res := i;
    break;
  end;
  Result := Res;
end;

destructor TMob.Destroy;
begin
  Look.Free;
  Spells.Free;
  Skills.Free;
  Helper.Free;
  Triggers.Free;
  inherited Destroy;
end;

{ TMobList }

procedure TMobList.Add(_number: integer);
var
  NewMob: TMob;
begin
  NewMob := TMob.Create(_number);
  inherited Add(NewMob);
end;

function TMobList.Convert: integer;
var
  tmpStr, WorkStr: string;
  i: integer;
  Spell: TSpell;
  Skill: TSkill;
begin
  StrCounter := -1;
  try
    Clear;
    repeat
      tmpStr := GetNextStr;
    until EndOfLines or ((tmpStr <> '') and (tmpStr[1] = '#'));
    if not EndOfLines then
    repeat
      System.Delete(tmpStr, 1, 1);
      Add(StrToInt(tmpStr));
      with TMob(Items[Count-1]) do
      begin
        tmpStr := GetNextStr; Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Imen := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Rod := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Dat := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Vin := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Tvor := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Pred := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Desc := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr;
        repeat
          tmpStr := GetNextStr;
          if tmpStr <> '~' then Look.Add(tmpStr);
        until tmpStr = '~';
        repeat
          tmpStr := GetNextStr;
        until tmpStr = '~';
        tmpStr := GetNextStr;
        Flags := ExtractWord(1, tmpStr, [' ']);
        if Flags = '0' then Flags := '';
        Flags := PrepareFlags(Flags);
        Affects := ExtractWord(2, tmpStr, [' ']);
        if Affects = '0' then Affects := '';
        Affects := PrepareFlags(Affects);
        Align := StrToInt(ExtractWord(3, tmpStr, [' ']));
        tmpStr := GetNextStr;
        Level := StrToInt(ExtractWord(1, tmpStr, [' ']));
        hitroll := StrToInt(ExtractWord(2, tmpStr, [' ']));
        AC := StrToInt(ExtractWord(3, tmpStr, [' ']));
        StrToDice(ExtractWord(4, tmpStr, [' ']), Hits);
        StrToDice(ExtractWord(5, tmpStr, [' ']), Damage);
        tmpStr := GetNextStr;
        StrToDice(ExtractWord(1, tmpStr, [' ']), Money);
        Exp := StrToInt(ExtractWord(2, tmpStr, [' ']));
        tmpStr := GetNextStr;
        PosLoad := StrToInt(ExtractWord(1, tmpStr, [' ']));
        PosDefault := StrToInt(ExtractWord(2, tmpStr, [' ']));
        Pol := StrToInt(ExtractWord(3, tmpStr, [' ']));
        repeat
          tmpStr := GetNextStr;
          WorkStr := ExtractWord(1, tmpStr, [':']);
          if WorkStr = 'BareHandAttack' then
           BareHandAttack := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Destination' then
           Destination := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Str' then
           Str := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Int' then
           Int := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Wis' then
           Wis := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Dex' then
           Dex := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Con' then
           Con := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Cha' then
           Cha := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'LikeWork' then
           LikeWork := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'MaxFactor' then
           MaxFactor := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'ExtraAttack' then
           ExtraAttack := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Class' then
           Model := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Size' then
           Size := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Height' then
           Height := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Weight' then
           Weight := StrToInt(ExtractWord(2, tmpStr, [' ']))
          else if WorkStr = 'Special_Bitvector' then
          begin
            Special := ExtractWord(2, tmpStr, [' ']);
            if Special = '0' then
             Special := ''
            else
             Special := PrepareFlags(Special);
          end else if WorkStr = 'Spell' then
          begin
            Spell.Number := StrToInt(ExtractWord(2, tmpStr, [' ']));
            i := FindSpellNum(Spell.Number);
            if i >=0 then
            begin
              Spell.Qty := TSpell(Spells.Items[i]).Qty + 1;
              Spells.Items[i] := TObject(Spell);
            end else
            begin
              Spell.Qty := 1;
              Spells.Add(TObject(Spell));
            end;
          end else if WorkStr = 'Skill' then
          begin
            Skill.Number := StrToInt(ExtractWord(2, tmpStr, [' ']));
            Skill.Percent := StrToInt(ExtractWord(3, tmpStr, [' ']));
            Skills.Add(TObject(Skill));
          end else if WorkStr = 'Helper' then
           Helper.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
        until WorkStr = 'E';
        tmpStr := GetNextStr;
        while (tmpStr<>'') and (tmpStr[1] = 'T') do
        begin
          Triggers.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
          tmpStr := GetNextStr;
        end;
      end;
      {...}
      while (not EndOfLines) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
       tmpStr := GetNextStr;
    until EndOfLines;
    Result := 0;
  except
    MessageDlg('     ' + IntToStr(StrCounter), mtError, [mbOk], 0);
    Result := StrCounter;
  end;
end;

procedure TMobList.SaveToFile(FileName: string);
var
  f: TextFile;
  i, j: integer;
  s1, s2: string;
begin
  try
    AssignFile(f, Filename); Rewrite(f);
    try
      WriteLn(f, '* BRusMUD mob file v1.0');
      for i := 0 to Count-1 do
       with TMob(Items[i]) do
      begin
        WriteLn(f, Format('#%d', [Number]));
        WriteLn(f, Alias + '~');
        WriteLn(f, Imen + '~');
        WriteLn(f, Rod + '~');
        WriteLn(f, Dat + '~');
        WriteLn(f, Vin + '~');
        WriteLn(f, Tvor + '~');
        WriteLn(f, Pred + '~');
        if Length(Desc) > 0 then WriteLn(f, Desc);
        WriteLn(f, '~');
        if Look.Count > 0 then WriteLn(f, Look.Text);
        WriteLn(f, '~');
        if Flags = '' then s1 := '0' else s1 := Flags;
        if Affects = '' then s2 := '0' else s2 := Affects;
        WriteLn(f, Format('%s %s %d E', [s1, s2, Align]));
        WriteLn(f, Format('%d %d %d %dd%d+%d %dd%d+%d',
         [Level, hitroll, AC, Hits.n, Hits.d, Hits.a, Damage.n, Damage.d, Damage.a]));
        WriteLn(f, Format('%dd%d+%d %d', [Money.n, Money.d, Money.a, Exp]));
        WriteLn(f, Format('%d %d %d', [PosLoad, PosDefault, Pol]));
        WriteLn(f, 'BareHandAttack: ', BareHandAttack);
        if Destination <> 0 then WriteLn(f, 'Destination: ', Destination);
        WriteLn(f, 'Str: ', Str);
        WriteLn(f, 'Int: ', Int);
        WriteLn(f, 'Wis: ', Wis);
        WriteLn(f, 'Dex: ', Dex);
        WriteLn(f, 'Con: ', Con);
        WriteLn(f, 'Cha: ', Cha);
        WriteLn(f, 'LikeWork: ', LikeWork);
        WriteLn(f, 'MaxFactor: ', MaxFactor);
        WriteLn(f, 'ExtraAttack: ', ExtraAttack);
        WriteLn(f, 'Class: ', Model);
        WriteLn(f, 'Size: ', Size);
        WriteLn(f, 'Height: ', Height);
        WriteLn(f, 'Weight: ', Weight);
        if Special <> '' then WriteLn(f, 'Special_Bitvector: ', Special);
        for j := 0 to Spells.Count-1 do
         with TSpell(Spells.Items[j]) do
          WriteLn(f, Format('Spell: %d %d', [Number, Qty]));
        for j := 0 to Skills.Count-1 do
         with TSkill(Skills.Items[j]) do
          WriteLn(f, Format('Skill: %d %d', [Number, Percent]));
        for j := 0 to Helper.Count-1 do
         WriteLn(f, 'Helper: ', integer(Helper.Items[j]));
        WriteLn(f, 'E');
        for j := 0 to Triggers.Count-1 do
         WriteLn(f, 'T ', integer(Triggers.Items[j]));
      end;
      WriteLn(f, '$'#13#10'$');
    finally
      CloseFile(f);
    end;
  except
    MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TMobList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TMob(Items[i]).Free;
  inherited Destroy;
end;

{ TShop }

constructor TShop.Create(_number: integer);
begin
  Number := _number;
  Goods := TList.Create;
  CoeffSell := 0;
  CoeffBuy := 0;
  CoeffChange := 0;
  GoodsBuy := TStringList.Create;
  GoodsChange := TStringList.Create;
  Msg1 := '';
  Msg2 := '';
  Msg3 := '';
  Msg4 := '';
  Msg5 := '';
  Msg6 := '';
  Msg7 := '';
  Emotion := 0;
  BitVector := '';
  Seller := 0;
  NotSellVector := '';
  Rooms := TList.Create;
  Open1 := 0;
  Close1 := 0;
  Open2 := 0;
  Close2 := 0;
  inherited Create;
end;

destructor TShop.Destroy;
begin
  Goods.Free;
  GoodsBuy.Free;
  GoodsChange.Free;
  Rooms.Free;
  inherited Destroy;
end;

{ TShopList }

procedure TShopList.Add(_number: integer);
var
  NewShop: TShop;
begin
  NewShop := TShop.Create(_number);
  inherited Add(NewShop);
end;

function TShopList.Convert: integer;
var
  tmpStr: string;
  i: integer;
  G: TGoods;
begin
  StrCounter := -1;
  try
    Clear;
    repeat
      tmpStr := GetNextStr;
    until EndOfLines or ((tmpStr <> '') and (tmpStr[1] = '#'));
    if not EndOfLines then
    repeat
      Add(StrToInt(Copy(tmpStr, 2, Length(tmpStr)-1)));
      with TShop(Items[Count-1]) do
      begin
        tmpStr := GetNextStr;
        while tmpStr <> '-1' do
        begin
          Goods.Add(TObject(StrToInt(tmpStr)));
          tmpStr := GetNextStr;
        end;
        tmpStr := GetNextStr; Val(DelSpace(tmpStr), CoeffSell, i);
        tmpStr := GetNextStr; Val(DelSpace(tmpStr), CoeffBuy, i);
        tmpStr := GetNextStr; // skip zero
        tmpStr := GetNextStr;
        while DelSpace(tmpStr) <> '-1' do
        begin
          G.Kind := StrToInt(DelSpace(tmpStr));
          G.Qty := 0;
          GoodsBuy.AddObject('', TObject(G));
          tmpStr := GetNextStr;
        end;
        tmpStr := GetNextStr; Msg1 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Msg2 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Msg3 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Msg4 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Msg5 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Msg6 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Msg7 := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr; Emotion := StrToInt(DelSpace(tmpStr));
        tmpStr := GetNextStr;
        if tmpStr = '0' then BitVector := ''
        else BitVector := PrepareFlags(tmpStr);
        tmpStr := GetNextStr; Seller := StrToInt(DelSpace(tmpStr));
        tmpStr := GetNextStr;
        if tmpStr = '0' then NotSellVector := ''
        else NotSellVector := PrepareFlags(tmpStr);
        tmpStr := GetNextStr; // skip zero
        tmpStr := GetNextStr;
        while DelSpace(tmpStr) <> '-1' do
        begin
          Rooms.Add(TObject(StrToInt(DelSpace(tmpStr))));
          tmpStr := GetNextStr;
        end;
        tmpStr := GetNextStr; Open1 := StrToInt(DelSpace(tmpStr));
        tmpStr := GetNextStr; Close1 := StrToInt(DelSpace(tmpStr));
        tmpStr := GetNextStr; Open2 := StrToInt(DelSpace(tmpStr));
        tmpStr := GetNextStr; Close2 := StrToInt(DelSpace(tmpStr));
      end;
      {...}
      while (not EndOfLines) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
       tmpStr := GetNextStr;
    until EndOfLines;
    Result := 0;
  except
    MessageDlg('     ' + IntToStr(StrCounter), mtError, [mbOk], 0);
    Result := StrCounter;
  end;
end;

procedure TShopList.SaveToFile(FileName: string);
var
  f: TextFile;
  i, j: integer;
begin
  try
    AssignFile(f, Filename); Rewrite(f);
    try
      WriteLn(f, '* BRusMUD shop file v1.0');
      for i := 0 to Count-1 do
       with TShop(Items[i]) do
      begin
        WriteLn(f, Format('#%d', [Number]));
        for j := 0 to Goods.Count-1 do
         WriteLn(f, integer(Goods.Items[j]));
        WriteLn(f, '-1');
        WriteLn(f, CoeffSell:0:2);
        WriteLn(f, CoeffBuy:0:2);
        WriteLn(f, CoeffChange:0:2);
        for j := 0 to GoodsBuy.Count-1 do
         with TGoods(GoodsBuy.Objects[j]) do
          WriteLn(f, Format('%d %d %s', [Kind, Qty, GoodsBuy[j]]));
        WriteLn(f, '-1');
        for j := 0 to GoodsChange.Count-1 do
         with TGoods(GoodsChange.Objects[j]) do
          WriteLn(f, Format('%d %d %s', [Kind, Qty, GoodsChange[j]]));
        WriteLn(f, '-1');
        WriteLn(f, Msg1 + '~');
        WriteLn(f, Msg2 + '~');
        WriteLn(f, Msg3 + '~');
        WriteLn(f, Msg4 + '~');
        WriteLn(f, Msg5 + '~');
        WriteLn(f, Msg6 + '~');
        WriteLn(f, Msg7 + '~');
        WriteLn(f, Emotion);
        if BitVector = '' then WriteLn(f, '0')
        else  WriteLn(f,  BitVector);
        WriteLn(f, Seller);
        if NotSellVector = '' then WriteLn(f, '0')
        else  WriteLn(f,  NotSellVector);
        for j := 0 to Rooms.Count-1 do
         WriteLn(f, integer(Rooms.Items[j]));
        WriteLn(f, '-1');
        WriteLn(f, Open1);
        WriteLn(f, Close1);
        WriteLn(f, Open2);
        WriteLn(f, Close2);
      end;
      WriteLn(f, '$'#13#10'$');
    finally
      CloseFile(f);
    end;
  except
    MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TShopList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TShop(Items[i]).Free;
  inherited Destroy;
end;

{ TRoom }

constructor TRoom.Create(_number: integer);
var
  i: integer;
  DefaultExit: TExit;
begin
  Number := _number;
  Name := '';
  Desc := TStringList.Create;
  Flags := '';
  Sector := 1;
  with DefaultExit do
  begin
    Name := '';
    Desc := '';
    Flag := 0;
    Key := -1;
    ExitRoom := 0;
    DefaultValue := 0;
  end;
  for i := 0 to 5 do
   Exits[i] := DefaultExit;
  Extra := TExtraDescList.Create;
  Triggers := TList.Create;
  inherited Create;
end;

destructor TRoom.Destroy;
begin
  Desc.Free;
  Extra.Free;
  Triggers.Free;
  inherited Destroy;
end;

{ TRoomList }

procedure TRoomList.Add(_number: integer);
var
  NewRoom: TRoom;
begin
  NewRoom := TRoom.Create(_number);
  inherited Add(NewRoom);
end;

function TRoomList.Convert: integer;
var
  tmpStr: string;
  ExtraDesc: TExtraDesc;
begin
  StrCounter := -1;
  try
    Clear;
    repeat
      tmpStr := GetNextStr;
    until EndOfLines or ((tmpStr <> '') and (tmpStr[1] = '#'));
    if not EndOfLines then
    repeat
      System.Delete(tmpStr, 1, 1);
      Add(StrToInt(tmpStr));
      with TRoom(Items[Count-1]) do
      begin
        tmpStr := GetNextStr; Name := Copy(tmpStr, 1, Length(tmpStr)-1);
        tmpStr := GetNextStr;
        while (tmpStr = '') or (tmpStr[Length(tmpStr)] <> '~') do
        begin
          Desc.Add(tmpStr);
          tmpStr := GetNextStr;
        end;
        tmpStr := Copy(tmpStr, 1, Length(tmpStr)-1);
        if tmpStr <> '' then
         Desc.Add(tmpStr);
        tmpStr := GetNextStr;
        Flags := ExtractWord(2, tmpStr, [' ']);
        if Flags = '0' then Flags := ''
        else Flags := PrepareFlags(Flags);
        Sector := StrToInt(ExtractWord(3, tmpStr, [' ']));
        tmpStr := GetNextStr;
        while (tmpStr<>'') and (tmpStr[1] = 'D') do
         with Exits[Ord(tmpStr[2])-$30] do
        begin
          tmpStr := GetNextStr; Desc := Copy(tmpStr, 1, Length(tmpStr)-1);
          tmpStr := GetNextStr; Name := Copy(tmpStr, 1, Length(tmpStr)-1);
          tmpStr := GetNextStr;
          Flag := StrToInt(ExtractWord(1, tmpStr, [' ']));
          Key := StrToInt(ExtractWord(2, tmpStr, [' ']));
          ExitRoom := StrToInt(ExtractWord(3, tmpStr, [' ']));
          tmpStr := GetNextStr;
        end;
        while (tmpStr<>'') and (tmpStr[1] = 'E') do
        begin
          ExtraDesc := TExtraDesc.Create;
          tmpStr := GetNextStr;
          ExtraDesc.Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
          tmpStr := GetNextStr;
          while tmpStr[Length(tmpStr)] <> '~' do
          begin
            ExtraDesc.Desc.Add(tmpStr);
            tmpStr := GetNextStr;
          end;
          tmpStr := Copy(tmpStr, 1, Length(tmpStr)-1);
          if tmpStr <> '' then
           ExtraDesc.Desc.Add(tmpStr);
          Extra.Add(ExtraDesc);
          tmpStr := GetNextStr;
        end;
        while (tmpStr<>'S') do tmpStr := GetNextStr;
        tmpStr := GetNextStr;
        while (tmpStr<>'') and (tmpStr[1] = 'T') do
        begin
          Triggers.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
          tmpStr := GetNextStr;
        end;
      end;
      {...}
      while (not EndOfLines) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
       tmpStr := GetNextStr;
    until EndOfLines;
    Result := 0;
  except
    MessageDlg('     ' + IntToStr(StrCounter), mtError, [mbOk], 0);
    Result := StrCounter;
  end;
end;

procedure TRoomList.SaveToFile(FileName: string);
var
  f: TextFile;
  i, j: integer;
  s: string;
begin
  try
    AssignFile(f, Filename); Rewrite(f);
    try
      WriteLn(f, '* BRusMUD room file v1.0');
      for i := 0 to Count-1 do
       with TRoom(Items[i]) do
      begin
        WriteLn(f, Format('#%d', [Number]));
        WriteLn(f, Name + '~');
        WriteLn(f, Desc.Text + '~');
        if Flags = '' then s := '0'
        else s := Flags;
        WriteLn(f, Format('%d %s %d', [Number div 100, s, Sector]));
        for j := 0 to 5 do
         with Exits[j] do
          if ExitRoom <> 0 then
        begin
          WriteLn(f, Format('D%d', [j]));
          WriteLn(f, Desc + '~');
          WriteLn(f, Name + '~');
          WriteLn(f, Format('%d %d %d', [Flag, Key, ExitRoom]));
        end;
        for j := 0 to Extra.Count-1 do
         with TExtraDesc(Extra.Items[j]) do
        begin
          WriteLn(f, 'E');
          WriteLn(f, Alias + '~');
          WriteLn(f, Desc.Text + '~');
        end;
        WriteLn(f, 'S');
        for j := 0 to Triggers.Count-1 do
         WriteLn(f, Format('T %d', [integer(Triggers.Items[j])]));
      end;
      WriteLn(f, '$'#13#10'$');
    finally
      CloseFile(f);
    end;
  except
    MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TRoomList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TRoom(Items[i]).Free;
  inherited Destroy;
end;

{ TZone }

constructor TZone.Create;
begin
  Number := 0;
  Name := '';
  RepopTime := 0;
  RepopType := 0;
  Items := TStringList.Create;
  inherited Create;
end;

procedure TZone.Clear;
begin
  Number := 0;
  Name := '';
  RepopTime := 0;
  RepopType := 0;
  Items.Clear;
end;

function TZone.Convert: integer;
var
  tmpStr: string;
  i: integer;
begin
  StrCounter := -1;
  try
    Clear;
    repeat
      tmpStr := GetNextStr;
    until EndOfLines or ((tmpStr <> '') and (tmpStr[1] = '#'));
    Val(Copy(tmpStr, 2, Length(tmpStr)-1), Number, i);
    tmpStr := GetNextStr;
    Name := Copy(tmpStr, 1, Length(tmpStr)-1);
    tmpStr := GetNextStr;
    tmpStr := GetNextStr;
    RepopTime := StrToInt(ExtractWord(2, tmpStr, [' ']));
    RepopType := StrToInt(ExtractWord(3, tmpStr, [' ']));
    tmpStr := GetNextStr;
    while (not EndOfLines) and (tmpStr <> 'S') do
    begin
      Items.Add(tmpStr);
      tmpStr := GetNextStr;
    end;
    Result := 0;
  except
    MessageDlg('     ' + IntToStr(StrCounter), mtError, [mbOk], 0);
    Result := StrCounter;
  end;
end;

procedure TZone.SaveToFile(FileName: string);
var
  f: TextFile;
  i: integer;
begin
  try
    AssignFile(f, Filename); Rewrite(f);
    try
      WriteLn(f, '* BRusMUD zone file v1.0');
      WriteLn(f, Format('#%d', [Number]));
      WriteLn(f, Name + '~');
      WriteLn(f, Format('%d %d %d', [Number*100+99, RepopTime, RepopType]));
      for i := 0 to Items.Count-1 do
       WriteLn(f, Items[i]);
      WriteLn(f, 'S'#13#10'$'#13#10'$');
    finally
      CloseFile(f);
    end;
  except
    MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TZone.Destroy;
begin
  Items.Free;
  inherited Destroy;
end;

end.
