unit vars;

interface

uses
  Classes, Dialogs, Windows, Graphics, ExtCtrls, StdCtrls, Messages;

const
  IniFileName = 'BRusMUDEd.ini';

  ExitNorth = 0;
  ExitEast  = 1;
  ExitSouth = 2;
  ExitWest  = 3;
  ExitUp    = 4;
  ExitDown  = 5;

  AntiExit: array [0..5] of byte = (2, 3, 0, 1, 5, 4);

  ExitsName: array [0..5] of string =
   (' ', ' ', ' ', ' ', '', '');

  Filter_TXT = 'Text files|*.txt|All files|*.*';
  Filter_ZON = 'Zone files|*.zon|All files|*.*';
  Filter_WLD = 'Room files|*.wld|All files|*.*';
  Filter_MAP = 'Map files|*.map|All files|*.*';
  Filter_MOB = 'Mob files|*.mob|All files|*.*';
  Filter_OBJ = 'Object files|*.obj|All files|*.*';
  Filter_SHP = 'Shop files|*.shp|All files|*.*';
  Filter_TRG = 'Trigger files|*.trg|All files|*.*';

  MinZoneX = 0;
  MaxZoneX = 99;
  MinZoneY = 0;
  MaxZoneY = 99;
  MinZoneZ = 1;
  MaxZoneZ = 9;

  mMobTrigger = 0;
  mObjectTrigger = 1;
  mRoomTrigger = 2;

  MaxRoomAddDesc = 10;

  MinutesInHour = 60;
  MinutesInDay = 60*24;

type
  TDice = record { Dice roll: nDd+a }
    n: byte;
    d: byte;
    a: smallint;
  end;

  TSkill = record
    Number: word;  { }
    Percent: smallint; {, %}
  end;

  TSpell = record
    Number: word;  { }
    Qty: smallint; { }
  end;

  TExtraDesc = class
    Alias: string;
    Desc: TStringList;
    constructor Create;
    destructor Destroy; override;
  end;

  TExtraDescList = class (TList)
    procedure Assign(Source : TExtraDescList);
    destructor Destroy; override;
  end;

  TAddParam = record
    Number: word;
    Value: smallint;
  end;

  TObj = class
    Number: integer;          {    }
    Alias: string;
    Imen, Rod, Dat, Vin, Tvor, Pred: string;
    Desc: string;
    Action: string;
    TrenSkill: smallint;
    MagicVector: string;
    MaxDurab, CurrDurab: integer;
    Material: word;
    Pol: byte;
    Timer: integer;
    Spell: smallint;
    SpellLevel: byte;
    Effects, Discomfort, Bans: string;
    Kind: byte;
    Flags, WearFlags: string;
    Param1: string;
    Param2, Param3, Param4: integer;
    Weight: integer;
    Price, RentInv, RentWear: integer;
    Extra: TExtraDescList;
    AddParam: TList;
    Triggers: TList;
    MaxInWorld: integer;
    constructor Create(_number: integer);
    procedure SetToEditor;
    procedure GetFromEditor;
    procedure Assign(Obj: TObj);
    destructor Destroy; override;
  end;

  TObjList = class (TList)
    function CheckNumber(_number: integer): boolean;
    function FindFreeNumber(ZoneNum: integer): byte;
    function FindItem(_number: integer): integer;
    function NumToName(_number: integer): string;
    procedure Add(_number: integer);
    procedure Refresh;
    procedure LoadFromFile(FileName: string);
    procedure FillSource(StrList: TStrings; ZoneNum: integer);
    procedure SaveToFile(FileName: string; ZoneNum: integer);
    destructor Destroy; override;
  end;

  TMob = class
    Number: integer;          {    }
    Alias: string;
    Imen, Rod, Dat, Vin, Tvor, Pred: string;
    Desc: string;
    Look: TStringList;
    Flags: string;
    Affects: string;
    Align: integer;
    Level: word;
    hitroll: shortint;
    AC: shortint;
    Hits: TDice;
    Damage: TDice;
    Money: TDice;
    Exp: integer;
    PosLoad, PosDefault: byte;
    Pol: byte;
    {additions}
    BareHandAttack: shortint;
    Destination: string;
    Str, Int, Wis, Dex, Con, Cha: byte;
    LikeWork: integer;
    MaxFactor: integer;
    ExtraAttack: word;
    Model: byte;
    Size: integer;
    Height: integer;
    Weight: integer;
    Special: string;
    Spells: TList;
    Skills: TList;
    Helper: TList;
    Triggers: TList;
    MaxInWorld: integer;
    constructor Create(_number: integer);
    procedure SetToEditor;
    procedure GetFromEditor;
    procedure Assign(Mob: TMob);
    function FindSpellNum(SpellNum: word): integer;
    function FindSpell(SpellNum: word): integer;
    function FindSkill(SkillNum: word): integer;
    destructor Destroy; override;
  end;

  TMobList = class (TList)
    function CheckNumber(_number: integer): boolean;
    function FindFreeNumber(ZoneNum: integer): byte;
    function FindItem(_number: integer): integer;
    function NumToName(_number: integer): string;
    procedure Add(_number: integer);
    procedure Refresh;
    procedure LoadFromFile(FileName: string);
    procedure FillSource(StrList: TStrings; ZoneNum: integer);
    procedure SaveToFile(FileName: string; ZoneNum: integer);
    destructor Destroy; override;
  end;

  TShop = class
    Number: integer;               {  }
    Goods: TList;                  { ,    }
    CoeffSell: real;               {   }
    CoeffBuy: real;                {   }
    CoeffChange: real;             {   }
    GoodsBuy: TStringList;         {   }
    GoodsChange: TStringList;      {   }
    Msg1: string;                  {     (%s   )}
    Msg2: string;                  {    }
    Msg3: string;                  {   }
    Msg4: string;                  {      }
    Msg5: string;                  {      }
    Msg6: string;                  {   (%d   )}
    Msg7: string;                  {    (%d   )}
    Emotion: integer;              {    }
    BitVector: string;             {}
    Seller: integer;               { ( )}
    NotSellVector: string;         {     }
    Rooms: TList;                  {   }
    Open1: byte;                   {  1}
    Close1: byte;                  {  1}
    Open2: byte;                   {  2}
    Close2: byte;                  {  2}
    constructor Create(_number: integer);
    procedure SetToEditor;
    procedure GetFromEditor;
    procedure Assign(Shop: TShop);
    destructor Destroy; override;
  end;

  TShopList = class (TList)
    function CheckNumber(_number: integer): boolean;
    function FindFreeNumber(ZoneNum: integer): byte;
    function FindItem(_number: integer): integer;
    procedure Add(_number: integer);
    procedure Refresh;
    procedure LoadFromFile(FileName: string);
    procedure FillSource(Str: TStrings; ZoneNum: integer);
    procedure SaveToFile(FileName: string; ZoneNum: integer);
    destructor Destroy; override;
  end;

  TTrigger = class
    Number: integer;
    Name: string;
    Model: byte;
    Flags: string;
    NumParam: integer;
    StrParam: string;
    Content: TStringList;
    constructor Create(_number: integer);
    procedure SetToEditor;
    procedure GetFromEditor;
    procedure Assign(Trigger: TTrigger);
    destructor Destroy; override;
  end;

  TTriggerList = class (TList)
    function CheckNumber(_number: integer): boolean;
    function FindFreeNumber(ZoneNum: integer): byte;
    function FindItem(_number: integer): integer;
    procedure Add(_number: integer);
    procedure Refresh;
    function NumToName(_number: integer): string;
    procedure LoadFromFile(FileName: string);
    procedure FillSource(Str: TStrings; ZoneNum: integer);
    procedure SaveToFile(FileName: string; ZoneNum: integer);
    destructor Destroy; override;
  end;

  TExit = record
    Name: string;                  { }
    Desc: string;                  {}
    Flag: byte;                    {}
    Key: integer;                  {}
    ExitRoom: integer;             {vnum   }
    DefaultValue: byte;            {  }
  end;

  TArrayExits = array [0..5] of TExit;

  TLinkObj = class
    Number: integer;
    IFFlag: boolean;
    Where: shortint;
    Prob: shortint;
    constructor Create(_number: integer);
  end;

  TLinkObjList = class (TList)
    procedure Assign(Source : TLinkObjList);
    destructor Destroy; override;
  end;

  TRoomObj = class
    Number: integer;
    IFFlag: boolean;
    Prob: shortint;
    Objects: TLinkObjList;
    constructor Create(_number: integer);
    procedure Assign(RoomObj: TRoomObj);
    destructor Destroy; override;
  end;

  TRoomObjList = class (TList)
    function FindItem(_number: integer): integer;
    destructor Destroy; override;
  end;

  TRoomMob = class
    Number: integer;
    IFFlag: boolean;
    Follow: integer;
    MaxQty: smallint;
    Objects: TLinkObjList;
    constructor Create(_number: integer);
    procedure Assign(RoomMob: TRoomMob);
    destructor Destroy; override;
  end;

  TRoomMobList = class (TList)
    function FindItem(_number: integer): integer;
    destructor Destroy; override;
  end;

  TRoom = class
    Number: integer;
    x, y, z: byte;
    Name: string;
    Desc: TStringList;
    Flags: string;
    Sector: byte;
    Exits: TArrayExits;
    Extra: TExtraDescList;
    Triggers: TList;
    Mobs: TRoomMobList;
    Objects: TRoomObjList;
    DelObjects: TList;
    constructor Create(_number: integer);
    procedure SetToEditor;
    procedure GetFromEditor;
    procedure Assign(Room: TRoom);
    destructor Destroy; override;
  end;

  TRoomList = class (TList)
    function CheckNumber(_number: integer): boolean;
    function FindFreeNumber(ZoneNum: integer): byte;
    function FindItem(_number: integer): integer;
    function NumToName(_number: integer): string;
    procedure Add(_number: integer);
    procedure Refresh;
    procedure LoadFromFile(FileName: string);
    procedure FillSource(Str: TStrings; ZoneNum: integer);
    procedure SaveToFile(FileName: string; ZoneNum: integer);
    procedure FillMap(ZoneNumber: integer);
    procedure DrawMiniMap(ZoneNum: integer);
    destructor Destroy; override;
  end;

  TZone = class
    Number: integer;
    Name: string;
    RepopTime: integer;
    RepopType: integer;
    Objects: TRoomObjList;
    DelMobs: TList;
    FileName_ZON: string;
    FileName_WLD: string;
    FileName_OBJ: string;
    FileName_MOB: string;
    FileName_SHP: string;
    FileName_TRG: string;
    constructor Create(_number: integer);
    procedure Clear;
    function LoadZone(FileName: string): boolean;
    function LoadFromFile(FileName: string): boolean;
    procedure FillSource(Str: TStrings);
    procedure SaveToFile(FileName: string);
    procedure SetToEditor;
    procedure GetFromEditor;
    destructor Destroy; override;
  end;

var
  ObjList: TObjList;
  MobList: TMobList;
  ShopList: TShopList;
  TriggerList: TTriggerList;
  RoomList: TRoomList;
  ZeroObj: TObj;
  ZeroMob: TMob;
  ZeroShop: TShop;
  ZeroTrig: TTrigger;
  ZeroRoom: TRoom;
  CurrentObj, CurrentMob, CurrentShop, CurrentTrigger, CurrentRoom: integer;
  CurrentZone, CurrentDoor, CurrentTimerType: integer;
  CurrentDesc: TStringList;
  CurrentExtraDesc: TExtraDescList;
  CurrentExits: TArrayExits;
  RoomAddDesc: array [1..MaxRoomAddDesc] of string[1];
  ZoneMap: array [MinZoneX..MaxZoneX, MinZoneY..MaxZoneY, MinZoneZ..MaxZoneZ] of integer;
  ShowErrMsg: boolean;

  FileCounter: integer;

function FileErrMsg(Code: integer): string;
procedure ReadF(const f: TextFile; var s: string);

procedure StrToDice(s: string; var Dice: TDice);
procedure DiceToStr(var s: string; Dice: TDice);

function StrToBool(s: string): boolean;
function BoolToStr(b: boolean): string;

implementation

uses
  main, effect, lists, SysUtils, StrUtils;

{ boolean & string }

function StrToBool(s: string): boolean;
begin
  Result := UpperCase(s) = 'TRUE';
end;

function BoolToStr(b: boolean): string;
begin
  if b then
   Result := 'True'
  else
   Result := 'False';
end;

{ Internal }

function PrepareFlags(s: string): string;
var
  i: integer;
  tStr: string;
begin
  tStr := '';
  for i := 1 to Length(s) do
   tStr := tStr + s[i] + '0';
  Result := tStr;
end;

function RCL(const s: string): string;
var
  i: integer;
begin
  i := Length(s);
  while (i > 0) and (s[i] in [#13, #10]) do Dec(i);
  Result := Copy(s, 1, i);
end;

{ Files }

function FileErrMsg(Code: integer): string;
begin
  case Code of
   -1: Result := #13#10'(   )';
   -2: Result := #13#10'(   )';
   else Result := Format(#13#10'(   %d)', [Code]);
  end;
end;

procedure ReadF(const f: TextFile; var s: string);
begin
  Inc(FileCounter);
  ReadLn(f, s);
end;

{ Dice}

procedure StrToDice(s: string; var Dice: TDice);
begin
  {NdD+A}
  Dice.n := StrToInt(ExtractWord(1, s, ['d']));
  Delete(s, 1, Pos('d', s));
  Dice.d := StrToInt(ExtractWord(1, s, ['+']));
  Dice.a := StrToInt(ExtractWord(2, s, ['+']));
end;

procedure DiceToStr(var s: string; Dice: TDice);
begin
  s := IntToStr(Dice.n) + 'd' + IntToStr(Dice.d) + '+' + IntToStr(Dice.a);
end;

{ TExtraDesc }

constructor TExtraDesc.Create;
begin
  inherited Create;
  Alias := '';
  Desc := TStringList.Create;
end;

destructor TExtraDesc.Destroy;
begin
  Desc.Free;
  inherited Destroy;
end;

{ TExtraDescList }

procedure TExtraDescList.Assign(Source : TExtraDescList);
var
  i: integer;
  ExtraDesc: TExtraDesc;
begin
  Clear;
  for i := 0 to Source.Count-1 do
  begin
    ExtraDesc := TExtraDesc.Create;
    ExtraDesc.Alias := TExtraDesc(Source.Items[i]).Alias;
    ExtraDesc.Desc.Assign(TExtraDesc(Source.Items[i]).Desc);
    Add(ExtraDesc);
  end;
end;

destructor TExtraDescList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TExtraDesc(Items[i]).Free;
  inherited Destroy;
end;

{ TObj }

constructor TObj.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  Alias := '';
  Imen := '';
  Rod := ''; Dat := ''; Vin := ''; Tvor := ''; Pred := '';
  Desc := '';
  Action := '';
  TrenSkill := 0;
  MagicVector := '';
  MaxDurab := 0; CurrDurab := 0;
  Material := 0;
  Pol := 0;
  Timer := 0;
  Spell := 0;
  SpellLevel := 0;
  Effects := ''; Discomfort := ''; Bans := '';
  Kind := 0;
  Flags := ''; WearFlags := '';
  Param1 := '';
  Param2 := 0; Param3 := 0; Param4 := 0;
  Weight := 0;
  Price := 0; RentInv := 0; RentWear := 0;
  Extra := TExtraDescList.Create;
  AddParam := TList.Create;
  Triggers := TList.Create;
  MaxInWorld := 0;
end;

procedure TObj.SetToEditor;
var
  i: integer;
begin
  with MainForm do
  begin
    O_AliasEdit.Text := Alias;
    O_Name1Edit.Text := Imen;
    O_Name2Edit.Text := Rod;
    O_Name3Edit.Text := Dat;
    O_Name4Edit.Text := Vin;
    O_Name5Edit.Text := Tvor;
    O_Name6Edit.Text := Pred;
    O_DescEdit.Text := Desc;
    O_AddDescEdit.Text := Self.Action;
    O_SkillComboBox.ItemIndex := O_SkillComboBox.Items.IndexOfObject(TObject(TrenSkill));
    O_WearComboBox.ItemIndex := O_WearComboBox.Items.IndexOfObject(TObject(MaxDurab));
    O_WearEdit.Value := CurrDurab;
    O_MaterialComboBox.ItemIndex := O_MaterialComboBox.Items.IndexOfObject(TObject(Material));
    O_KindComboBox.ItemIndex := O_KindComboBox.Items.IndexOfObject(TObject(Pol));
    if (Timer mod MinutesInDay) = 0 then
    begin
      O_TimerEdit.Value := Timer div MinutesInDay;
      CurrentTimerType := 2;
    end else
    if (Timer mod MinutesInHour) = 0 then
    begin
      O_TimerEdit.Value := Timer div MinutesInHour;
      CurrentTimerType := 1;
    end else
    begin
      O_TimerEdit.Value := Timer;
      CurrentTimerType := 0;
    end;
    O_TimerComboBox.ItemIndex := CurrentTimerType;
    if Spell = 0 then Spell := -1;
    O_SpellComboBox.ItemIndex := O_SpellComboBox.Items.IndexOfObject(TObject(Spell));
    O_SpellLevelEdit.Value := SpellLevel;
    SetCheckListBox(O_EffectCheckListBox, Flags);
    SetCheckListBox(O_NoUseCheckListBox, Discomfort);
    SetCheckListBox(O_NoTakeCheckListBox, Bans);
    O_TypeComboBox.ItemIndex := O_TypeComboBox.Items.IndexOfObject(TObject(Kind));
    SetCheckListBox(O_AffectCheckListBox, Effects);
    SetCheckListBox(O_WearCheckListBox, WearFlags);
    case Kind of
     1:
     begin
       O_TLightEdit.Value := StrToInt(Param1);
     end;
     2:
     begin
       O_T2LevelEdit.Value := StrToInt(Param1);
       if Param2 = 0 then Param2 := -1;
       O_T2Spell1ComboBox.ItemIndex := O_T2Spell1ComboBox.Items.IndexOfObject(TObject(Param2));
       if Param3 = 0 then Param3 := -1;
       O_T2Spell2ComboBox.ItemIndex := O_T2Spell2ComboBox.Items.IndexOfObject(TObject(Param3));
       if Param4 = 0 then Param4 := -1;
       O_T2Spell3ComboBox.ItemIndex := O_T2Spell3ComboBox.Items.IndexOfObject(TObject(Param4));
     end;
     3:
     begin
       O_T3LevelEdit.Value := StrToInt(Param1);
       O_T3QtyEdit.Value := Param2;
       O_T3CountEdit.Value := Param3;
       if Param4 = 0 then Param4 := -1;
       O_T3SpellComboBox.ItemIndex := O_T3SpellComboBox.Items.IndexOfObject(TObject(Param4));
     end;
     4:
     begin
       O_T4LevelEdit.Value := StrToInt(Param1);
       O_T4QtyEdit.Value := Param2;
       O_T4CountEdit.Value := Param3;
       if Param4 = 0 then Param4 := -1;
       O_T4SpellComboBox.ItemIndex := O_T4SpellComboBox.Items.IndexOfObject(TObject(Param4));
     end;
     5:
     begin
       O_T5Dice1Edit.Value := Param2;
       O_T5Dice2Edit.Value := Param3;
       O_T5ComboBox.ItemIndex := O_T5ComboBox.Items.IndexOfObject(TObject(Param4));
     end;
     9:
     begin
       O_TACEdit.Value := StrToInt(Param1);
       O_TArmourEdit.Value := Param2;
     end;
     10:
     begin
       O_T10LevelEdit.Value := StrToInt(Param1);
       if Param2 = 0 then Param2 := -1;
       O_T10Spell1ComboBox.ItemIndex := O_T10Spell1ComboBox.Items.IndexOfObject(TObject(Param2));
       if Param3 = 0 then Param3 := -1;
       O_T10Spell2ComboBox.ItemIndex := O_T10Spell2ComboBox.Items.IndexOfObject(TObject(Param3));
       if Param4 = 0 then Param4 := -1;
       O_T10Spell3ComboBox.ItemIndex := O_T10Spell3ComboBox.Items.IndexOfObject(TObject(Param4));
     end;
     15:
     begin
       O_T15Edit1.Value := StrToInt(Param1);
       if (Param2 and 1) <> 0 then O_T15CheckListBox.Checked[0] := True
       else  O_T15CheckListBox.Checked[0] := False;
       if (Param2 and 2) <> 0 then O_T15CheckListBox.Checked[1] := True
       else  O_T15CheckListBox.Checked[1] := False;
       if (Param2 and 4) <> 0 then O_T15CheckListBox.Checked[2] := True
       else  O_T15CheckListBox.Checked[2] := False;
       if (Param2 and 8) <> 0 then O_T15CheckListBox.Checked[3] := True
       else  O_T15CheckListBox.Checked[3] := False;
       O_T15Edit2.Value := Param3;
     end;
     17:
     begin
       O_T17Edit1.Value := StrToInt(Param1);
       O_T17Edit2.Value := Param2;
       O_T17ComboBox.ItemIndex := O_T17ComboBox.Items.IndexOfObject(TObject(Param3));
       O_T17Edit3.Value := Param4;
     end;
     19:
     begin
       O_T19Edit1.Value := StrToInt(Param1);
       O_T19Edit2.Value := Param2;
     end;
     20:
     begin
       O_T20Edit.Value := StrToInt(Param1);
     end;
     23:
     begin
       O_T23Edit1.Value := StrToInt(Param1);
       O_T23Edit2.Value := Param2;
       O_T23ComboBox.ItemIndex := O_T23ComboBox.Items.IndexOfObject(TObject(Param3));
       O_T23Edit3.Value := Param4;
     end;
     24:
     begin
       if Param2 = 0 then Param2 := -1;
       O_T24SpellComboBox.ItemIndex := O_T24SpellComboBox.Items.IndexOfObject(TObject(Param2));
//       O_T24LevelEdit.Value := Param1;
//       O_T24ClassComboBox.ItemIndex := O_T24ClassComboBox.Items.IndexOfObject(TObject(Param3));
     end;
     25:
     begin
       SetCheckListBox(O_T25CheckListBox1, MagicVector);
       i := StrToInt(Param1);
       O_T25Edit3.Value := i and $FF;
       O_T25Edit4.Value := i shr 8;
       O_T25Edit1.Value := Param2;
       O_T25Edit2.Value := Param3;
     end;
    end; {case}
    O_TypeComboBoxChange(nil);
    O_WeightEdit.Value := Weight;
    O_PriceEdit.Value := Price;
    O_RentInvEdit.Value := RentInv;
    O_RentWearEdit.Value := RentWear;
    O_MaxInWorldEdit.Value := MaxInWorld;
    O_DescComboBox.Items.Clear;
    for i := 0 to Extra.Count-1 do
     O_DescComboBox.Items.Add(TExtraDesc(Extra.Items[i]).Alias);
    CurrentExtraDesc.Assign(Extra);
    if Extra.Count > 0 then
     O_DescComboBox.ItemIndex := 0
    else
     O_DescComboBox.ItemIndex := -1;
    O_DescComboBoxChange(nil);
    O_EffectStringGrid.RowCount := AddParam.Count;
    if AddParam.Count = 0 then
    begin
      O_EffectStringGrid.Cells[0, 0] := '';
      O_EffectStringGrid.Cells[1, 0] := '';
    end else
    for i := 0 to AddParam.Count-1 do
     with TAddParam(AddParam.Items[i]) do
    begin
      O_EffectStringGrid.Cells[0, i] := IntToStr(Value);
      O_EffectStringGrid.Objects[0, i] := TObject(Number);
      O_EffectStringGrid.Cells[1, i] := EffectForm.ComboBox.Items[EffectForm.ComboBox.Items.IndexOfObject(TObject(Number))];
    end;
    O_TriggerListBox.Clear;
    for i := 0 to Triggers.Count-1 do
     O_TriggerListBox.Items.AddObject(TriggerList.NumToName(integer(Triggers.Items[i])), Triggers.Items[i]);
  end;
end;

procedure TObj.GetFromEditor;
var
  i: integer;
  AddP: TAddParam;
begin
  with MainForm do
  begin
    Alias := O_AliasEdit.Text;
    Imen := O_Name1Edit.Text;
    Rod := O_Name2Edit.Text;
    Dat := O_Name3Edit.Text;
    Vin := O_Name4Edit.Text;
    Tvor := O_Name5Edit.Text;
    Pred := O_Name6Edit.Text;
    Desc := O_DescEdit.Text;
    Self.Action := O_AddDescEdit.Text;
    TrenSkill := integer(O_SkillComboBox.Items.Objects[O_SkillComboBox.ItemIndex]);
    MaxDurab := integer(O_WearComboBox.Items.Objects[O_WearComboBox.ItemIndex]);
    CurrDurab := O_WearEdit.Value;
    Material := integer(O_MaterialComboBox.Items.Objects[O_MaterialComboBox.ItemIndex]);
    Pol := integer(O_KindComboBox.Items.Objects[O_KindComboBox.ItemIndex]);
    case CurrentTimerType of
     0: {minutes}
      Timer := O_TimerEdit.Value;
     1: {hours}
      Timer := O_TimerEdit.Value*MinutesInHour;
     2: {days}
      Timer := O_TimerEdit.Value*MinutesInDay;
    end;
    Spell := integer(O_SpellComboBox.Items.Objects[O_SpellComboBox.ItemIndex]);
    SpellLevel := O_SpellLevelEdit.Value;
    GetCheckListBox(O_EffectCheckListBox, Flags);
    GetCheckListBox(O_NoUseCheckListBox, Discomfort);
    GetCheckListBox(O_NoTakeCheckListBox, Bans);
    Kind := integer(O_TypeComboBox.Items.Objects[O_TypeComboBox.ItemIndex]);
    GetCheckListBox(O_AffectCheckListBox, Effects);
    GetCheckListBox(O_WearCheckListBox, WearFlags);
    case Kind of
     1:
     begin
       Param1 := IntToStr(O_TLightEdit.Value);
     end;
     2:
     begin
       Param1 := IntToStr(O_T2LevelEdit.Value);
       Param2 := integer(O_T2Spell1ComboBox.Items.Objects[O_T2Spell1ComboBox.ItemIndex]);
       Param3 := integer(O_T2Spell2ComboBox.Items.Objects[O_T2Spell2ComboBox.ItemIndex]);
       Param4 := integer(O_T2Spell3ComboBox.Items.Objects[O_T2Spell3ComboBox.ItemIndex]);
     end;
     3:
     begin
       Param1 := IntToStr(O_T3LevelEdit.Value);
       Param2 := O_T3QtyEdit.Value;
       Param3 := O_T3CountEdit.Value;
       Param4 := integer(O_T3SpellComboBox.Items.Objects[O_T3SpellComboBox.ItemIndex]);
     end;
     4:
     begin
       Param1 := IntToStr(O_T4LevelEdit.Value);
       Param2 := O_T4QtyEdit.Value;
       Param3 := O_T4CountEdit.Value;
       Param4 := integer(O_T4SpellComboBox.Items.Objects[O_T4SpellComboBox.ItemIndex]);
     end;
     5:
     begin
       Param2 := O_T5Dice1Edit.Value;
       Param3 := O_T5Dice2Edit.Value;
       Param4 := integer(O_T5ComboBox.Items.Objects[O_T5ComboBox.ItemIndex]);
     end;
     9:
     begin
       Param1 := IntToStr(O_TACEdit.Value);
       Param2 := O_TArmourEdit.Value;
     end;
     10:
     begin
       Param1 := IntToStr(O_T10LevelEdit.Value);
       Param2 := integer(O_T10Spell1ComboBox.Items.Objects[O_T10Spell1ComboBox.ItemIndex]);
       Param3 := integer(O_T10Spell2ComboBox.Items.Objects[O_T10Spell2ComboBox.ItemIndex]);
       Param4 := integer(O_T10Spell3ComboBox.Items.Objects[O_T10Spell3ComboBox.ItemIndex]);
     end;
     15:
     begin
       Param1 := IntToStr(O_T15Edit1.Value);
       Param2 := 0;
       if O_T15CheckListBox.Checked[0] then Param2 := Param2 or 1;
       if O_T15CheckListBox.Checked[1] then Param2 := Param2 or 2;
       if O_T15CheckListBox.Checked[2] then Param2 := Param2 or 4;
       if O_T15CheckListBox.Checked[3] then Param2 := Param2 or 8;
       Param3 := O_T15Edit2.Value;
     end;
     17:
     begin
       Param1 := IntToStr(O_T17Edit1.Value);
       Param2 := O_T17Edit2.Value;
       Param3 := integer(O_T17ComboBox.Items.Objects[O_T17ComboBox.ItemIndex]);
       Param4 := O_T17Edit3.Value;
     end;
     19:
     begin
       Param1 := IntToStr(O_T19Edit1.Value);
       Param2 := O_T19Edit2.Value;
     end;
     20:
     begin
       Param1 := IntToStr(O_T20Edit.Value);
     end;
     23:
     begin
       Param1 := IntToStr(O_T23Edit1.Value);
       Param2 := O_T23Edit2.Value;
       Param3 := integer(O_T23ComboBox.Items.Objects[O_T23ComboBox.ItemIndex]);
       Param4 := O_T23Edit3.Value;
     end;
     24:
     begin
       Param2 := integer(O_T24SpellComboBox.Items.Objects[O_T24SpellComboBox.ItemIndex]);
//       Param1 := O_T24LevelEdit.Value;
//       Param3 := integer(O_T24ClassComboBox.Items.Objects[O_T24ClassComboBox.ItemIndex]);
     end;
     25:
     begin
       GetCheckListBox(O_T25CheckListBox1, MagicVector);
       Param1 := IntToStr(O_T25Edit3.Value or (O_T25Edit4.Value shl 8));
       Param2 := O_T25Edit1.Value;
       Param3 := O_T25Edit2.Value;
       Param4 := 0;
     end;
    end; {case}
    Weight := O_WeightEdit.Value;
    Price := O_PriceEdit.Value;
    RentInv := O_RentInvEdit.Value;
    RentWear := O_RentWearEdit.Value;
    MaxInWorld := O_MaxInWorldEdit.Value;
    Extra.Assign(CurrentExtraDesc);
    AddParam.Clear;
    for i := 0 to O_EffectStringGrid.RowCount-1 do
     if O_EffectStringGrid.Cells[0, i] <> '' then
    begin
      AddP.Number := integer(O_EffectStringGrid.Objects[0, i]);
      AddP.Value := StrToInt(O_EffectStringGrid.Cells[0, i]);
      AddParam.Add(TObject(AddP));
    end;
    Triggers.Clear;
    for i := 0 to O_TriggerListBox.Items.Count-1 do
     Triggers.Add(O_TriggerListBox.Items.Objects[i]);
  end;
  {set in common list}
  i := ObjListForm.ListBox1.Items.IndexOfObject(TObject(Number));
  if i >= 0 then
   ObjListForm.ListBox1.Items[i] := ObjList.NumToName(Number)
  else
   ObjListForm.ListBox1.Items.AddObject(ObjList.NumToName(Number), TObject(Number));
end;

procedure TObj.Assign(Obj: TObj);
var
  i: integer;
begin
  Alias := Obj.Alias;
  Imen := Obj.Imen;
  Rod := Obj.Rod;
  Dat := Obj.Dat;
  Vin := Obj.Vin;
  Tvor := Obj.Tvor;
  Pred := Obj.Pred;
  Desc := Obj.Desc;
  Action := Obj.Action;
  TrenSkill := Obj.TrenSkill;
  MagicVector := Obj.MagicVector;
  MaxDurab := Obj.MaxDurab; CurrDurab := Obj.CurrDurab;
  Material := Obj.Material;
  Pol := Obj.Pol;
  Timer := Obj.Timer;
  Spell := Obj.Spell;
  SpellLevel := Obj.SpellLevel;
  Effects := Obj.Effects; Discomfort := Obj.Discomfort; Bans := Obj.Bans;
  Kind := Obj.Kind;
  Flags := Obj.Flags; WearFlags := Obj.WearFlags;
  Param1 := Obj.Param1;
  Param2 := Obj.Param2; Param3 := Obj.Param3; Param4 := Obj.Param4;
  Weight := Obj.Weight;
  Price := Obj.Price; RentInv := Obj.RentInv; RentWear := Obj.RentWear;
  MaxInWorld := Obj.MaxInWorld;
  Extra.Assign(Obj.Extra);
  AddParam.Clear;
  for i := 0 to Obj.AddParam.Count-1 do
   AddParam.Add(TObject(integer(Obj.AddParam.Items[i])));
  Triggers.Clear;
  for i := 0 to Obj.Triggers.Count-1 do
   Triggers.Add(TObject(integer(Obj.Triggers.Items[i])));
end;

destructor TObj.Destroy;
begin
  Extra.Free;
  AddParam.Free;
  Triggers.Free;
  inherited Destroy;
end;

{ TObjList }

function TObjList.CheckNumber(_number: integer): boolean;
var
  i: integer;
  b: boolean;
begin
  b := True;
  for i := 0 to Count-1 do
   if TObj(Items[i]).Number = _number then
    b := False;
  Result := b;
end;

function TObjList.FindFreeNumber(ZoneNum: integer): byte;
var
  i: integer;
begin
  for i := 0 to 99 do
   if CheckNumber(ZoneNum*100 + i) then
    break;
  Result := i;
end;

function TObjList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TObj(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

function TObjList.NumToName(_number: integer): string;
var
  i: integer;
  s: string;
begin
  i := FindItem(_number);
  s := IntToStr(_number);
  if i >= 0 then s := s + ' ' + TObj(Items[i]).Imen;
  Result := s;
end;

procedure TObjList.Add(_number: integer);
var
  NewObj: TObj;
begin
  NewObj := TObj.Create(_number);
  inherited Add(NewObj);
end;

procedure TObjList.Refresh;
var
  i: integer;
begin
  ObjListForm.ListBox1.Clear;
  for i := 0 to Count-1 do
   with TObj(Items[i]) do
    ObjListForm.ListBox1.Items.AddObject(IntToStr(Number) + '  ' + Imen, TObject(Number));
end;

procedure TObjList.LoadFromFile(FileName: string);
var
  f: TextFile;
  tmpStr: string;
  i, j, ItemNumber: integer;
  AddPar: TAddParam;
  ExtraDesc: TExtraDesc;
begin
  try
    FileCounter := -1;
    AssignFile(f, Filename); Reset(f);
    try
      FileCounter := 0;
      repeat
        ReadF(f, tmpStr);
      until Eof(f) or ((tmpStr <> '') and (tmpStr[1] = '#'));
      if not Eof(f) then
      repeat
        System.Delete(tmpStr, 1, 1);
        ItemNumber := FindItem(StrToInt(tmpStr));
        if ItemNumber < 0 then
        begin
          Add(StrToInt(tmpStr));
          ItemNumber := Count-1;
        end;
        with TObj(Items[ItemNumber]) do
        begin
          ReadF(f, tmpStr); Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Imen := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Rod := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Dat := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Vin := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Tvor := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Pred := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Desc := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr);
          Action := '';
          if tmpStr <> '~' then
          begin
            if tmpStr[Length(tmpStr)] = '~' then
             Action := Copy(tmpStr, 1, Length(tmpStr)-1)
            else
            begin
              Action := tmpStr;
              ReadF(f, tmpStr);
            end;
          end;
          ReadF(f, tmpStr);
          MagicVector := ExtractWord(1, tmpStr, [' ']);
          Val(MagicVector, TrenSkill, i);
          if MagicVector = '0' then MagicVector := '';
          if i <> 0 then TrenSkill := 0;
          MaxDurab := StrToInt(ExtractWord(2, tmpStr, [' ']));
          CurrDurab := StrToInt(ExtractWord(3, tmpStr, [' ']));
          Material := StrToInt(ExtractWord(4, tmpStr, [' ']));
          ReadF(f, tmpStr);
          Pol := StrToInt(ExtractWord(1, tmpStr, [' ']));
          Timer := StrToInt(ExtractWord(2, tmpStr, [' ']));
          Spell := StrToInt(ExtractWord(3, tmpStr, [' ']));
          SpellLevel := StrToInt(ExtractWord(4, tmpStr, [' ']));
          ReadF(f, tmpStr);
          Effects := ExtractWord(1, tmpStr, [' ']);
          if Effects = '0' then Effects := '';
          Discomfort := ExtractWord(2, tmpStr, [' ']);
          if Discomfort = '0' then Discomfort := '';
          Bans := ExtractWord(3, tmpStr, [' ']);
          if Bans = '0' then Bans := '';
          ReadF(f, tmpStr);
          Kind := StrToInt(ExtractWord(1, tmpStr, [' ']));
          if (Kind = 25) and (TrenSkill <> 0) then
          begin
            MagicVector := '';
            for i := 0 to 5 do
             if (TrenSkill and (1 shl i)) <> 0 then
              MagicVector := MagicVector + Chr(i+Ord('a')) + '0';
            TrenSkill := 0;
          end;
          Flags := ExtractWord(2, tmpStr, [' ']);
          if Flags = '0' then Flags := '';
          WearFlags := ExtractWord(3, tmpStr, [' ']);
          if WearFlags = '0' then WearFlags := '';
          ReadF(f, tmpStr);
          Param1 := ExtractWord(1, tmpStr, [' ']);
          Val(Param1, i, j);
          if j <> 0 then
          begin
            j := 0;
            for i := 1 to (Length(Param1) div 2) do
             j := j + (1 shl (Ord(Param1[i*2-1]) - Ord('a')));
            Param1 := IntToStr(j);
          end;
          Param2 := StrToInt(ExtractWord(2, tmpStr, [' ']));
          Param3 := StrToInt(ExtractWord(3, tmpStr, [' ']));
          Param4 := StrToInt(ExtractWord(4, tmpStr, [' ']));
          ReadF(f, tmpStr);
          Weight := StrToInt(ExtractWord(1, tmpStr, [' ']));
          Price := StrToInt(ExtractWord(2, tmpStr, [' ']));
          RentInv := StrToInt(ExtractWord(3, tmpStr, [' ']));
          RentWear := StrToInt(ExtractWord(4, tmpStr, [' ']));
          ReadF(f, tmpStr);
          Extra.Clear;
          while (tmpStr <> '') and (tmpStr[1] = 'E') do
          begin
            ExtraDesc := TExtraDesc.Create;
            ReadF(f, tmpStr);
            ExtraDesc.Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
            ReadF(f, tmpStr);
            while tmpStr <> '~' do
            begin
              if (tmpStr <> '') and (tmpStr[Length(tmpStr)] = '~') then
              begin
                ExtraDesc.Desc.Add(Copy(tmpStr, 1, Length(tmpStr)-1));
                tmpStr := '~'
              end else
              begin
                ExtraDesc.Desc.Add(tmpStr);
                ReadF(f, tmpStr);
              end;
            end;
            Extra.Add(ExtraDesc);
            ReadF(f, tmpStr);
          end;
          AddParam.Clear;
          while (tmpStr<>'') and (tmpStr[1] = 'A') do
          begin
            ReadF(f, tmpStr);
            AddPar.Number := StrToInt(ExtractWord(1, tmpStr, [' ']));
            AddPar.Value := StrToInt(ExtractWord(2, tmpStr, [' ']));
            AddParam.Add(TObject(AddPar));
            ReadF(f, tmpStr);
          end;
          Triggers.Clear;
          while (tmpStr<>'') and (tmpStr[1] = 'T') do
          begin
            Triggers.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
            ReadF(f, tmpStr);
          end;
        end;
        while (not Eof(f)) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
         ReadF(f, tmpStr);
      until Eof(f);
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
  except
    if ShowErrMsg then
     MessageDlg('   '#13#10': ' + FileName +
                 FileErrMsg(FileCounter), mtError, [mbOk], 0);
  end;
end;

procedure TObjList.FillSource(StrList: TStrings; ZoneNum: integer);
var
  i, j, Num: integer;
  s1, s2: string;
begin
  StrList.Clear;
  StrList.Add('* BRusMUD object file v1.0');
  for i := 0 to 99 do
  begin
    Num := FindItem(ZoneNum*100 + i);
    if Num >= 0 then
     with TObj(Items[Num]) do
   begin
    StrList.Add(Format('#%d', [Number]));
    StrList.Add(Alias + '~');
    StrList.Add(Imen + '~');
    StrList.Add(Rod + '~');
    StrList.Add(Dat + '~');
    StrList.Add(Vin + '~');
    StrList.Add(Tvor + '~');
    StrList.Add(Pred + '~');
    StrList.Add(Desc + '~');
    if Length(Action) > 0 then StrList.Add(Action);
    StrList.Add('~');
    if Kind = 25 then
    begin
      if MagicVector = '' then s1 := '0'
      else s1 := MagicVector;
    end else
     s1 := IntToStr(TrenSkill);
    StrList.Add(Format('%s %d %d %d', [s1, MaxDurab, CurrDurab, Material]));
    StrList.Add(Format('%d %d %d %d', [Pol, Timer, Spell, SpellLevel]));
    s1 := '';
    if Effects = '' then s1 := '0 '
    else s1 := Effects + ' ';
    if Discomfort = '' then s1 := s1 + '0 '
    else s1 := s1 + Discomfort + ' ';
    if Bans = '' then StrList.Add(s1 + '0')
    else StrList.Add(s1 + Bans);
    if Flags = '' then s1 := '0'
    else s1 := Flags;
    if WearFlags = '' then s2 := '0'
    else s2 := WearFlags;
    StrList.Add(Format('%d %s %s', [Kind, s1, s2]));
    if Param1 = '' then s1 := '0'
    else s1 := Param1;
    StrList.Add(Format('%s %d %d %d', [s1, Param2, Param3, Param4]));
    StrList.Add(Format('%d %d %d %d', [Weight, Price, RentInv, RentWear]));
    for j := 0 to Extra.Count-1 do
     with TExtraDesc(Extra.Items[j]) do
    begin
      StrList.Add('E');
      StrList.Add(Alias + '~');
      StrList.Add(Desc.Text + '~');
    end;
    for j := 0 to AddParam.Count-1 do
     with TAddParam(AddParam.Items[j]) do
    begin
      StrList.Add('A');
      StrList.Add(Format('%d %d', [Number, Value]));
    end;
    for j := 0 to Triggers.Count-1 do
     StrList.Add(Format('T %d', [integer(Triggers.Items[j])]));
   end;
  end;
  StrList.Add('$'#13#10'$');
end;

procedure TObjList.SaveToFile(FileName: string; ZoneNum: integer);
var
  Str: TStringList;
begin
  try
    Str := TStringList.Create;
    FillSource(Str, ZoneNum);
    Str.SaveToFile(FileName);
    Str.Free;
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TObjList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TObj(Items[i]).Free;
  inherited Destroy;
end;

{ TMob }

constructor TMob.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  Alias := '';
  Imen := '';
  Rod := ''; Dat := ''; Vin := ''; Tvor := ''; Pred := '';
  Desc := '';
  Look := TStringList.Create;
  Flags := '';
  Affects := '';
  Align := 0;
  Level := 0;
  hitroll := 0;
  AC := 0;
  Hits.n := 0; Hits.d := 0; Hits.a := 0;
  Damage.n := 0; Damage.d := 0; Damage.a := 0;
  Money.n := 0; Money.d := 0; Money.a := 0;
  Exp := 0;
  PosLoad := 0; PosDefault := 0;
  Pol := 0;
  BareHandAttack := 0;
  Destination := '';
  Str := 0; Int := 0; Wis := 0; Dex := 0; Con := 0; Cha := 0;
  LikeWork := 0;
  MaxFactor := 0;
  ExtraAttack := 0;
  Model := 0;
  Size := 0;
  Height := 0;
  Weight := 0;
  Special := '';
  Spells := TList.Create;
  Skills := TList.Create;
  Helper := TList.Create;
  Triggers := TList.Create;
end;

function TMob.FindSpellNum(SpellNum: word): integer;
var
  i, Res: integer;
begin
  Res := -1;
  for i := 0 to Spells.Count-1 do
   if TSpell(Spells.Items[i]).Number = SpellNum then
  begin
    Res := i;
    break;
  end;
  Result := Res;
end;

function TMob.FindSpell(SpellNum: word): integer;
var
  i, Res: integer;
begin
  Res := 0;
  for i := 0 to Spells.Count-1 do
   if TSpell(Spells.Items[i]).Number = SpellNum then
  begin
    Res := TSpell(Spells.Items[i]).Qty;
    break;
  end;
  Result := Res;
end;

function TMob.FindSkill(SkillNum: word): integer;
var
  i, Res: integer;
begin
  Res := 0;
  for i := 0 to Skills.Count-1 do
   if TSkill(Skills.Items[i]).Number = SkillNum then
  begin
    Res := TSkill(Skills.Items[i]).Percent;
    break;
  end;
  Result := Res;
end;

procedure TMob.SetToEditor;
var
  i: integer;
begin
  with MainForm do
  begin
    M_AliasEdit.Text := Alias;
    M_Name1Edit.Text := Imen;
    M_Name2Edit.Text := Rod;
    M_Name3Edit.Text := Dat;
    M_Name4Edit.Text := Vin;
    M_Name5Edit.Text := Tvor;
    M_Name6Edit.Text := Pred;
    M_DescEdit.Text := Desc;
    if Look.Count > 0 then
     M_FullDescEdit.Text := Look[0]
    else
     M_FullDescEdit.Text := '';
    CurrentDesc.Assign(Look);
    SetCheckListBox(M_ActionCheckListBox, Flags);
    SetCheckListBox(M_AffectCheckListBox, Affects);
    M_AlignComboBox.ItemIndex := M_AlignComboBox.Items.IndexOfObject(TObject(Self.Align));
    M_LevelComboBox.ItemIndex := M_LevelComboBox.Items.IndexOfObject(TObject(Level));
    M_Thac0ComboBox.ItemIndex := M_Thac0ComboBox.Items.IndexOfObject(TObject(hitroll));
    M_ACComboBox.ItemIndex := M_ACComboBox.Items.IndexOfObject(TObject(AC));
    M_HP1Edit.Text := IntToStr(Hits.n);
    M_HP2Edit.Text := IntToStr(Hits.d);
    M_HP3Edit.Text := IntToStr(Hits.a);
    if Hits.a < 0 then
     M_HPBtn.Caption := '-';
    M_Damage1Edit.Text := IntToStr(Damage.n);
    M_Damage2Edit.Text := IntToStr(Damage.d);
    M_Damage3Edit.Text := IntToStr(Damage.a);
    if Damage.a < 0 then
     M_DamageBtn.Caption := '-';
    M_Money1Edit.Text := IntToStr(Money.n);
    M_Money2Edit.Text := IntToStr(Money.d);
    M_Money3Edit.Text := IntToStr(Money.a);
    if Money.a < 0 then
     M_MoneyBtn.Caption := '-';
    M_ExpEdit.Value := Exp;
    M_State1ComboBox.ItemIndex := M_State1ComboBox.Items.IndexOfObject(TObject(PosLoad));
    M_State2ComboBox.ItemIndex := M_State2ComboBox.Items.IndexOfObject(TObject(PosDefault));
    M_SexComboBox.ItemIndex := M_SexComboBox.Items.IndexOfObject(TObject(Pol));
    M_BareHandComboBox.ItemIndex := M_BareHandComboBox.Items.IndexOfObject(TObject(BareHandAttack));
    M_DestinationEdit.Text := Destination;
    M_StrComboBox.ItemIndex := M_StrComboBox.Items.IndexOfObject(TObject(Str));
    M_IntComboBox.ItemIndex := M_IntComboBox.Items.IndexOfObject(TObject(Int));
    M_WisComboBox.ItemIndex := M_WisComboBox.Items.IndexOfObject(TObject(Wis));
    M_DexComboBox.ItemIndex := M_DexComboBox.Items.IndexOfObject(TObject(Dex));
    M_ConComboBox.ItemIndex := M_ConComboBox.Items.IndexOfObject(TObject(Con));
    M_ChaComboBox.ItemIndex := M_ChaComboBox.Items.IndexOfObject(TObject(Cha));
    M_LikeWorkComboBox.ItemIndex := M_LikeWorkComboBox.Items.IndexOfObject(TObject(LikeWork));
    M_MaxFactorComboBox.ItemIndex := M_MaxFactorComboBox.Items.IndexOfObject(TObject(MaxFactor));
    M_ExtraAttackComboBox.ItemIndex := M_ExtraAttackComboBox.Items.IndexOfObject(TObject(ExtraAttack));
    M_ClassComboBox.ItemIndex := M_ClassComboBox.Items.IndexOfObject(TObject(Model));
    M_SizeEdit.Value := Size;
    M_HeightEdit.Value := Self.Height;
    M_Weight.Value := Weight;
    M_MaxInWorldEdit.Value := MaxInWorld;
    SetCheckListBox(M_SpecialCheckListBox, Special);
    for i := 0 to M_SpellsStringGrid.RowCount-1 do
      M_SpellsStringGrid.Cells[3, i] := IntToStr(FindSpell(integer(M_SpellsStringGrid.Objects[0, i])));
    for i := 0 to M_SkillsStringGrid.RowCount-1 do
      M_SkillsStringGrid.Cells[2, i] := IntToStr(FindSkill(integer(M_SkillsStringGrid.Objects[0, i])));
    M_HelperListBox.Items.Clear;
    for i := 0 to Helper.Count-1 do
     M_HelperListBox.Items.Add(IntToStr(integer(Helper.Items[i])));
    M_TriggerListBox.Clear;
    for i := 0 to Triggers.Count-1 do
     M_TriggerListBox.Items.AddObject(TriggerList.NumToName(integer(Triggers.Items[i])), Triggers.Items[i]);
  end;
end;

procedure TMob.GetFromEditor;
var
  i: integer;
  Spell: TSpell;
  Skill: TSkill;
begin
  with MainForm do
  begin
    Alias := M_AliasEdit.Text;
    Imen := M_Name1Edit.Text;
    Rod := M_Name2Edit.Text;
    Dat := M_Name3Edit.Text;
    Vin := M_Name4Edit.Text;
    Tvor := M_Name5Edit.Text;
    Pred := M_Name6Edit.Text;
    Desc := M_DescEdit.Text;
    Look.Assign(CurrentDesc);
    GetCheckListBox(M_ActionCheckListBox, Flags);
    GetCheckListBox(M_AffectCheckListBox, Affects);
    Self.Align := integer(M_AlignComboBox.Items.Objects[M_AlignComboBox.ItemIndex]);
    Level := integer(M_LevelComboBox.Items.Objects[M_LevelComboBox.ItemIndex]);
    hitroll := integer(M_Thac0ComboBox.Items.Objects[M_Thac0ComboBox.ItemIndex]);
    AC := integer(M_ACComboBox.Items.Objects[M_ACComboBox.ItemIndex]);
    Hits.n := StrToInt(M_HP1Edit.Text);
    Hits.d := StrToInt(M_HP2Edit.Text);
    Hits.a := StrToInt(M_HP3Edit.Text);
    if M_HPBtn.Caption = '-' then
     Hits.a := -Hits.a;
    Damage.n := StrToInt(M_Damage1Edit.Text);
    Damage.d := StrToInt(M_Damage2Edit.Text);
    Damage.a := StrToInt(M_Damage3Edit.Text);
    if M_DamageBtn.Caption = '-' then
     Damage.a := -Damage.a;
    Money.n := StrToInt(M_Money1Edit.Text);
    Money.d := StrToInt(M_Money2Edit.Text);
    Money.a := StrToInt(M_Money3Edit.Text);
    if M_MoneyBtn.Caption = '-' then
     Money.a := -Money.a;
    Exp := M_ExpEdit.Value;
    PosLoad := integer(M_State1ComboBox.Items.Objects[M_State1ComboBox.ItemIndex]);
    PosDefault := integer(M_State2ComboBox.Items.Objects[M_State2ComboBox.ItemIndex]);
    Pol := integer(M_SexComboBox.Items.Objects[M_SexComboBox.ItemIndex]);
    BareHandAttack := integer(M_BareHandComboBox.Items.Objects[M_BareHandComboBox.ItemIndex]);
    Destination := M_DestinationEdit.Text;
    Str := integer(M_StrComboBox.Items.Objects[M_StrComboBox.ItemIndex]);
    Int := integer(M_IntComboBox.Items.Objects[M_IntComboBox.ItemIndex]);
    Wis := integer(M_WisComboBox.Items.Objects[M_WisComboBox.ItemIndex]);
    Dex := integer(M_DexComboBox.Items.Objects[M_DexComboBox.ItemIndex]);
    Con := integer(M_ConComboBox.Items.Objects[M_ConComboBox.ItemIndex]);
    Cha := integer(M_ChaComboBox.Items.Objects[M_ChaComboBox.ItemIndex]);
    LikeWork := integer(M_LikeWorkComboBox.Items.Objects[M_LikeWorkComboBox.ItemIndex]);
    MaxFactor := integer(M_MaxFactorComboBox.Items.Objects[M_MaxFactorComboBox.ItemIndex]);
    ExtraAttack := integer(M_ExtraAttackComboBox.Items.Objects[M_ExtraAttackComboBox.ItemIndex]);
    Model := integer(M_ClassComboBox.Items.Objects[M_ClassComboBox.ItemIndex]);
    Size := M_SizeEdit.Value;
    Self.Height := M_HeightEdit.Value;
    Weight := M_Weight.Value;
    MaxInWorld := M_MaxInWorldEdit.Value;
    GetCheckListBox(M_SpecialCheckListBox, Special);
    Spells.Clear;
    for i := 0 to M_SpellsStringGrid.RowCount-1 do
     if StrToInt(M_SpellsStringGrid.Cells[3, i]) <> 0 then
    begin
      Spell.Number := integer(M_SpellsStringGrid.Objects[0, i]);
      Spell.Qty := StrToInt(M_SpellsStringGrid.Cells[3, i]);
      Spells.Add(TObject(Spell));
    end;
    Skills.Clear;
    for i := 0 to M_SkillsStringGrid.RowCount-1 do
     if StrToInt(M_SkillsStringGrid.Cells[2, i]) <> 0 then
    begin
      Skill.Number := integer(M_SkillsStringGrid.Objects[0, i]);
      Skill.Percent := StrToInt(M_SkillsStringGrid.Cells[2, i]);
      Skills.Add(TObject(Skill));
    end;
    Helper.Clear;
    for i := 0 to M_HelperListBox.Items.Count-1 do
     Helper.Add(TObject(StrToInt(M_HelperListBox.Items[i])));
    Triggers.Clear;
    for i := 0 to M_TriggerListBox.Items.Count-1 do
     Triggers.Add(M_TriggerListBox.Items.Objects[i]);
  end;
  {set in common list}
  i := MobListForm.ListBox1.Items.IndexOfObject(TObject(Number));
  if i >= 0 then
   MobListForm.ListBox1.Items[i] := MobList.NumToName(Number)
  else
   MobListForm.ListBox1.Items.AddObject(MobList.NumToName(Number), TObject(Number));
end;

procedure TMob.Assign(Mob: TMob);
var
  i: integer;
begin
  Alias := Mob.Alias;
  Imen := Mob.Imen;
  Rod := Mob.Rod;
  Dat := Mob.Dat;
  Vin := Mob.Vin;
  Tvor := Mob.Tvor;
  Pred := Mob.Pred;
  Desc := Mob.Desc;
  Look.Assign(Mob.Look);
  Flags := Mob.Flags;
  Affects := Mob.Affects;
  Align := Mob.Align;
  Level := Mob.Level;
  hitroll := Mob.hitroll;
  AC := Mob.AC;
  Hits := Mob.Hits;
  Damage := Mob.Damage;
  Money := Mob.Money;
  Exp := Mob.Exp;
  PosLoad := Mob.PosLoad;
  PosDefault := Mob.PosDefault;
  Pol := Mob.Pol;
  {additions}
  BareHandAttack := Mob.BareHandAttack;
  Destination := Mob.Destination;
  Str := Mob.Str;
  Int := Mob.Int;
  Wis := Mob.Wis;
  Dex := Mob.Dex;
  Con := Mob.Con;
  Cha := Mob.Cha;
  LikeWork := Mob.LikeWork;
  MaxFactor := Mob.MaxFactor;
  ExtraAttack := Mob.ExtraAttack;
  Model := Mob.Model;
  Size := Mob.Size;
  Height := Mob.Height;
  Weight := Mob.Weight;
  MaxInWorld := Mob.MaxInWorld;
  Special := Mob.Special;
  Spells.Clear;
  for i := 0 to Mob.Spells.Count-1 do
   Spells.Add(TObject(integer(Mob.Spells.Items[i])));
  Skills.Clear;
  for i := 0 to Mob.Skills.Count-1 do
   Skills.Add(TObject(integer(Mob.Skills.Items[i])));
  Helper.Clear;
  for i := 0 to Mob.Helper.Count-1 do
   Helper.Add(TObject(integer(Mob.Helper.Items[i])));
  Triggers.Clear;
  for i := 0 to Mob.Triggers.Count-1 do
   Triggers.Add(TObject(integer(Mob.Triggers.Items[i])));
end;

destructor TMob.Destroy;
begin
  Look.Free;
  Spells.Free;
  Skills.Free;
  Helper.Free;
  Triggers.Free;
  inherited Destroy;
end;

{ TMobList }

function TMobList.CheckNumber(_number: integer): boolean;
var
  i: integer;
  b: boolean;
begin
  b := True;
  for i := 0 to Count-1 do
   if TMob(Items[i]).Number = _number then
    b := False;
  Result := b;
end;

function TMobList.FindFreeNumber(ZoneNum: integer): byte;
var
  i: integer;
begin
  for i := 0 to 99 do
   if CheckNumber(ZoneNum*100 + i) then
    break;
  Result := i;
end;

function TMobList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TMob(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

function TMobList.NumToName(_number: integer): string;
var
  i: integer;
  s: string;
begin
  i := FindItem(_number);
  s := IntToStr(_number);
  if i >= 0 then s := s + ' ' + TMob(Items[i]).Imen;
  Result := s;
end;

procedure TMobList.Add(_number: integer);
var
  NewMob: TMob;
begin
  NewMob := TMob.Create(_number);
  inherited Add(NewMob);
end;

procedure TMobList.Refresh;
var
  i: integer;
begin
  MobListForm.ListBox1.Clear;
  for i := 0 to Count-1 do
   with TMob(Items[i]) do
    MobListForm.ListBox1.Items.AddObject(IntToStr(Number) + '  ' + Imen, TObject(Number));
end;

procedure TMobList.LoadFromFile(FileName: string);
var
  f: TextFile;
  tmpStr, WorkStr: string;
  i, ItemNumber: integer;
  Spell: TSpell;
  Skill: TSkill;
begin
  try
    FileCounter := -1;
    AssignFile(f, Filename); Reset(f);
    try
      FileCounter := 0;
      repeat
        ReadF(f, tmpStr);
      until Eof(f) or ((tmpStr <> '') and (tmpStr[1] = '#'));
      if not Eof(f) then
      repeat
        System.Delete(tmpStr, 1, 1);
        ItemNumber := FindItem(StrToInt(tmpStr));
        if ItemNumber < 0 then
        begin
          Add(StrToInt(tmpStr));
          ItemNumber := Count-1;
        end;
        with TMob(Items[ItemNumber]) do
        begin
          ReadF(f, tmpStr); Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Imen := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Rod := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Dat := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Vin := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Tvor := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Pred := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr);
          Desc := '';
          if tmpStr <> '~' then
          begin
            if tmpStr[Length(tmpStr)] = '~' then
             Desc := Copy(tmpStr, 1, Length(tmpStr)-1)
            else
            begin
              Desc := tmpStr;
              ReadF(f, tmpStr);
            end;
          end;
          ReadF(f, tmpStr);
          Look.Clear;
          while tmpStr <> '~' do
          begin
            if (tmpStr <> '') and (tmpStr[Length(tmpStr)] = '~') then
            begin
              Look.Add(Copy(tmpStr, 1, Length(tmpStr)-1));
              tmpStr := '~';
            end else
            begin
              Look.Add(tmpStr);
              ReadF(f, tmpStr);
            end;
          end;
          ReadF(f, tmpStr);
          Flags := ExtractWord(1, tmpStr, [' ']);
          if Flags = '0' then Flags := '';
          Affects := ExtractWord(2, tmpStr, [' ']);
          if Affects = '0' then Affects := '';
          Align := StrToInt(ExtractWord(3, tmpStr, [' ']));
          ReadF(f, tmpStr);
          Level := StrToInt(ExtractWord(1, tmpStr, [' ']));
          hitroll := StrToInt(ExtractWord(2, tmpStr, [' ']));
          AC := StrToInt(ExtractWord(3, tmpStr, [' ']));
          StrToDice(ExtractWord(4, tmpStr, [' ']), Hits);
          StrToDice(ExtractWord(5, tmpStr, [' ']), Damage);
          ReadF(f, tmpStr);
          StrToDice(ExtractWord(1, tmpStr, [' ']), Money);
          Exp := StrToInt(ExtractWord(2, tmpStr, [' ']));
          ReadF(f, tmpStr);
          PosLoad := StrToInt(ExtractWord(1, tmpStr, [' ']));
          PosDefault := StrToInt(ExtractWord(2, tmpStr, [' ']));
          Pol := StrToInt(ExtractWord(3, tmpStr, [' ']));
          Spells.Clear;
          Skills.Clear;
          Helper.Clear;
          Destination := '';
          repeat
            ReadF(f, tmpStr);
            WorkStr := ExtractWord(1, tmpStr, [':']);
            if WorkStr = 'BareHandAttack' then
             BareHandAttack := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Destination' then
            begin
              if Destination = '' then
               Destination := ExtractWord(2, tmpStr, [' '])
              else
               Destination := Destination + '/' + ExtractWord(2, tmpStr, [' ']);
            end
            else if WorkStr = 'Str' then
             Str := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Int' then
             Int := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Wis' then
             Wis := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Dex' then
             Dex := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Con' then
             Con := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Cha' then
             Cha := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'LikeWork' then
             LikeWork := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'MaxFactor' then
             MaxFactor := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'ExtraAttack' then
             ExtraAttack := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Class' then
             Model := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Size' then
             Size := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Height' then
             Height := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Weight' then
             Weight := StrToInt(ExtractWord(2, tmpStr, [' ']))
            else if WorkStr = 'Special_Bitvector' then
             Special := ExtractWord(2, tmpStr, [' '])
            else if WorkStr = 'Spell' then
            begin
              Spell.Number := StrToInt(ExtractWord(2, tmpStr, [' ']));
              i := FindSpellNum(Spell.Number);
              if i >= 0 then
              begin
                Spell.Qty := TSpell(Spells.Items[i]).Qty + 1;
                Spells.Items[i] := TObject(Spell);
              end else
              begin
                Spell.Qty := 1;
                Spells.Add(TObject(Spell));
              end;
            end else if WorkStr = 'Skill' then
            begin
              Skill.Number := StrToInt(ExtractWord(2, tmpStr, [' ']));
              Skill.Percent := StrToInt(ExtractWord(3, tmpStr, [' ']));
              Skills.Add(TObject(Skill));
            end else if WorkStr = 'Helper' then
             Helper.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
          until WorkStr = 'E';
          ReadF(f, tmpStr);
          Triggers.Clear;
          while (tmpStr<>'') and (tmpStr[1] = 'T') do
          begin
            Triggers.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
            ReadF(f, tmpStr);
          end;
        end;
        while (not Eof(f)) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
         ReadF(f, tmpStr);
      until Eof(f);
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
  except
//    Clear;
    if ShowErrMsg then
     MessageDlg('   '#13#10': ' + FileName +
                 FileErrMsg(FileCounter), mtError, [mbOk], 0);
  end;
end;

procedure TMobList.FillSource(StrList: TStrings; ZoneNum: integer);
var
  i, j, k, Num: integer;
  s1, s2: string;
begin
  StrList.Clear;
  StrList.Add('* BRusMUD mob file v1.0');
  for i := 0 to 99 do
  begin
    Num := FindItem(ZoneNum*100 + i);
    if Num >= 0 then
     with TMob(Items[Num]) do
   begin
    StrList.Add(Format('#%d', [Number]));
    StrList.Add(Alias + '~');
    StrList.Add(Imen + '~');
    StrList.Add(Rod + '~');
    StrList.Add(Dat + '~');
    StrList.Add(Vin + '~');
    StrList.Add(Tvor + '~');
    StrList.Add(Pred + '~');
    if Length(Desc) > 0 then StrList.Add(Desc);
    StrList.Add('~');
    StrList.Add(Look.Text + '~');
    if Flags = '' then s1 := '0' else s1 := Flags;
    if Affects = '' then s2 := '0' else s2 := Affects;
    StrList.Add(Format('%s %s %d E', [s1, s2, Align]));
    StrList.Add(Format('%d %d %d %dd%d+%d %dd%d+%d',
       [Level, hitroll, AC, Hits.n, Hits.d, Hits.a, Damage.n, Damage.d, Damage.a]));
    StrList.Add(Format('%dd%d+%d %d', [Money.n, Money.d, Money.a, Exp]));
    StrList.Add(Format('%d %d %d', [PosLoad, PosDefault, Pol]));
    StrList.Add(Format('BareHandAttack: %d', [BareHandAttack]));
    s1 := Destination;
    while s1 <> '' do
    begin
      j := Pos('/', s1);
      if j > 0 then
      begin
        StrList.Add('Destination: ' + Copy(s1, 1, j-1));
        System.Delete(s1, 1, j);
      end else
      begin
        StrList.Add('Destination: ' + s1);
        s1 := '';
      end;
    end;
    StrList.Add('Str: ' + IntToStr(Str));
    StrList.Add('Int: ' + IntToStr(Int));
    StrList.Add('Wis: ' + IntToStr(Wis));
    StrList.Add('Dex: ' + IntToStr(Dex));
    StrList.Add('Con: ' + IntToStr(Con));
    StrList.Add('Cha: ' + IntToStr(Cha));
    StrList.Add('LikeWork: ' + IntToStr(LikeWork));
    StrList.Add('MaxFactor: ' + IntToStr(MaxFactor));
    StrList.Add('ExtraAttack: ' + IntToStr(ExtraAttack));
    StrList.Add('Class: ' + IntToStr(Model));
    StrList.Add('Size: ' + IntToStr(Size));
    StrList.Add('Height: ' + IntToStr(Height));
    StrList.Add('Weight: ' + IntToStr(Weight));
    if Special <> '' then StrList.Add('Special_Bitvector: ' + Special);
    for j := 0 to Spells.Count-1 do
     with TSpell(Spells.Items[j]) do
      for k := 1 to Qty do
       StrList.Add(Format('Spell: %d', [Number]));
    for j := 0 to Skills.Count-1 do
     with TSkill(Skills.Items[j]) do
      StrList.Add(Format('Skill: %d %d', [Number, Percent]));
    for j := 0 to Helper.Count-1 do
     StrList.Add('Helper: ' + IntToStr(integer(Helper.Items[j])));
    StrList.Add('E');
    for j := 0 to Triggers.Count-1 do
     StrList.Add('T ' + IntToStr(integer(Triggers.Items[j])));
   end;
  end;
  StrList.Add('$'#13#10'$');
end;

procedure TMobList.SaveToFile(FileName: string; ZoneNum: integer);
var
  Str: TStringList;
begin
  try
    Str := TStringList.Create;
    FillSource(Str, ZoneNum);
    Str.SaveToFile(FileName);
    Str.Free;
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TMobList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TMob(Items[i]).Free;
  inherited Destroy;
end;

{ TShop }

constructor TShop.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  Goods := TList.Create;
  CoeffSell := 1.0;
  CoeffBuy := 1.0;
  CoeffChange := 1.0;
  GoodsBuy := TStringList.Create;
  GoodsChange := TStringList.Create;
  Msg1 := '';
  Msg2 := '';
  Msg3 := '';
  Msg4 := '';
  Msg5 := '';
  Msg6 := '';
  Msg7 := '';
  Emotion := 0;
  BitVector := '';
  Seller := 0;
  NotSellVector := '';
  Rooms := TList.Create;
  Open1 := 0;
  Close1 := 0;
  Open2 := 0;
  Close2 := 0;
end;

procedure TShop.SetToEditor;
var
  i: integer;
begin
  with MainForm do
  begin
    S_SellListBox.Clear;
    for i := 0 to Goods.Count-1 do
     S_SellListBox.Items.AddObject(ObjList.NumToName(integer(Goods[i])), Goods[i]);
    S_ProfitSellEdit.Value := Round(CoeffSell * 100);
    S_ProfitBuyEdit.Value := Round(CoeffBuy * 100);
    S_ProfitChangeEdit.Value := Round(CoeffChange * 100);
    S_BuyListBox.Clear;
    for i := 0 to GoodsBuy.Count-1 do
     S_BuyListBox.Items.AddObject(GoodsBuy[i], GoodsBuy.Objects[i]);
    S_ChangeListBox.Clear;
    for i := 0 to GoodsChange.Count-1 do
     S_ChangeListBox.Items.AddObject(GoodsChange[i], GoodsChange.Objects[i]);
    S_Msg1Edit.Text := Msg1;
    S_Msg2Edit.Text := Msg2;
    S_Msg3Edit.Text := Msg3;
    S_Msg4Edit.Text := Msg4;
    S_Msg5Edit.Text := Msg5;
    S_Msg6Edit.Text := Msg6;
    S_Msg7Edit.Text := Msg7;
    S_EmotionEdit.Value := Emotion;
    SetCheckListBox(S_BitVCheckListBox, BitVector);
    S_SellerEdit.Value := Seller;
    SetCheckListBox(S_WhoCheckListBox, NotSellVector);
    S_RoomListBox.Clear;
    for i := 0 to Rooms.Count-1 do
     S_RoomListBox.Items.AddObject(RoomList.NumToName(integer(Rooms[i])), Rooms[i]);
    S_Open1ComboBox.ItemIndex := S_Open1ComboBox.Items.IndexOfObject(TObject(Open1));
    S_Close1ComboBox.ItemIndex := S_Close1ComboBox.Items.IndexOfObject(TObject(Close1));
    S_Open2ComboBox.ItemIndex := S_Open2ComboBox.Items.IndexOfObject(TObject(Open2));
    S_Close2ComboBox.ItemIndex := S_Close2ComboBox.Items.IndexOfObject(TObject(Close2));
  end;
end;

procedure TShop.GetFromEditor;
var
  i: integer;
begin
  with MainForm do
  begin
    Goods.Clear;
    for i := 0 to S_SellListBox.Items.Count-1 do
     Goods.Add(S_SellListBox.Items.Objects[i]);
    CoeffSell := S_ProfitSellEdit.Value / 100;
    CoeffBuy := S_ProfitBuyEdit.Value / 100;
    CoeffChange := S_ProfitChangeEdit.Value / 100;
    GoodsBuy.Clear;
    with S_BuyListBox do
     for i := 0 to Items.Count-1 do
      GoodsBuy.AddObject(Items[i], Items.Objects[i]);
    GoodsChange.Clear;
    with S_ChangeListBox do
     for i := 0 to Items.Count-1 do
      GoodsChange.AddObject(Items[i], Items.Objects[i]);
    Msg1 := S_Msg1Edit.Text;
    Msg2 := S_Msg2Edit.Text;
    Msg3 := S_Msg3Edit.Text;
    Msg4 := S_Msg4Edit.Text;
    Msg5 := S_Msg5Edit.Text;
    Msg6 := S_Msg6Edit.Text;
    Msg7 := S_Msg7Edit.Text;
    Emotion := S_EmotionEdit.Value;
    GetCheckListBox(S_BitVCheckListBox, BitVector);
    Seller := S_SellerEdit.Value;
    GetCheckListBox(S_WhoCheckListBox, NotSellVector);
    Rooms.Clear;
    for i := 0 to S_RoomListBox.Items.Count-1 do
     Rooms.Add(S_RoomListBox.Items.Objects[i]);
    Open1 := integer(S_Open1ComboBox.Items.Objects[S_Open1ComboBox.ItemIndex]);
    Close1 := integer(S_Close1ComboBox.Items.Objects[S_Close1ComboBox.ItemIndex]);
    Open2 := integer(S_Open2ComboBox.Items.Objects[S_Open2ComboBox.ItemIndex]);
    Close2 := integer(S_Close2ComboBox.Items.Objects[S_Close2ComboBox.ItemIndex]);
  end;
  {set in common list}
  i := ShopListForm.ListBox1.Items.IndexOfObject(TObject(Number));
  if i < 0 then
   ShopListForm.ListBox1.Items.AddObject(IntToStr(Number), TObject(Number));
end;

procedure TShop.Assign(Shop: TShop);
var
  i: integer;
begin
  Goods.Clear;
  for i := 0 to Shop.Goods.Count-1 do
   Goods.Add(TObject(integer(Shop.Goods.Items[i])));
  CoeffSell := Shop.CoeffSell;
  CoeffBuy := Shop.CoeffBuy;
  CoeffChange := Shop.CoeffChange;
  GoodsBuy.Assign(Shop.GoodsBuy);
  GoodsChange.Assign(Shop.GoodsChange);
  Msg1 := Shop.Msg1;
  Msg2 := Shop.Msg2;
  Msg3 := Shop.Msg3;
  Msg4 := Shop.Msg4;
  Msg5 := Shop.Msg5;
  Msg6 := Shop.Msg6;
  Msg7 := Shop.Msg7;
  Emotion := Shop.Emotion;
  BitVector := Shop.BitVector;
  Seller := Shop.Seller;
  NotSellVector := Shop.NotSellVector;
  Rooms.Clear;
  for i := 0 to Shop.Rooms.Count-1 do
   Rooms.Add(TObject(integer(Shop.Rooms.Items[i])));
  Open1 := Shop.Open1;
  Close1 := Shop.Close1;
  Open2 := Shop.Open2;
  Close2 := Shop.Close2;
end;

destructor TShop.Destroy;
begin
  Goods.Free;
  GoodsBuy.Free;
  GoodsChange.Free;
  Rooms.Free;
  inherited Destroy;
end;

{ TShopList }

function TShopList.CheckNumber(_number: integer): boolean;
var
  i: integer;
  b: boolean;
begin
  b := True;
  for i := 0 to Count-1 do
   if TShop(Items[i]).Number = _number then
    b := False;
  Result := b;
end;

function TShopList.FindFreeNumber(ZoneNum: integer): byte;
var
  i: integer;
begin
  for i := 0 to 99 do
   if CheckNumber(ZoneNum*100 + i) then
    break;
  Result := i;
end;

function TShopList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TShop(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

procedure TShopList.Add(_number: integer);
var
  NewShop: TShop;
begin
  NewShop := TShop.Create(_number);
  inherited Add(NewShop);
end;

procedure TShopList.Refresh;
var
  i: integer;
begin
  ShopListForm.ListBox1.Clear;
  for i := 0 to Count-1 do
   with TShop(Items[i]) do
    ShopListForm.ListBox1.Items.AddObject(IntToStr(Number), TObject(Number));
end;

procedure TShopList.LoadFromFile(FileName: string);
var
  f: TextFile;
  tmpStr: string;
  i, j, ItemNumber: integer;
  NewFormat: boolean;
begin
  try
    FileCounter := -1;
    AssignFile(f, Filename); Reset(f);
    try
      FileCounter := 0;
      repeat
        ReadF(f, tmpStr);
      until Eof(f) or ((tmpStr <> '') and (tmpStr[1] = '#'));
      if not Eof(f) then
      repeat
        i := 2;
        while (i <= Length(tmpStr)) and (tmpStr[i] in ['0'..'9']) do
         Inc(i);
        if tmpStr[i] = '~' then NewFormat := False
        else NewFormat := True;
        tmpStr := Copy(tmpStr, 2, i-2);
        ItemNumber := FindItem(StrToInt(tmpStr));
        if ItemNumber < 0 then
        begin
          Add(StrToInt(tmpStr));
          ItemNumber := Count-1;
        end;
        with TShop(Items[ItemNumber]) do
        begin
          ReadF(f, tmpStr);
          Goods.Clear;
          while tmpStr <> '-1' do
          begin
            Goods.Add(TObject(StrToInt(tmpStr)));
            ReadF(f, tmpStr);
          end;
          ReadF(f, tmpStr); Val(tmpStr, CoeffSell, i);
          ReadF(f, tmpStr); Val(tmpStr, CoeffBuy, i);
          if NewFormat then
           ReadF(f, tmpStr); Val(tmpStr, CoeffChange, i);
          ReadF(f, tmpStr);
          GoodsBuy.Clear;
          while tmpStr <> '-1' do
          begin
            if tmpStr[Length(tmpStr)] = '~' then
            begin
              System.Delete(tmpStr, Length(tmpStr), 1);
              i := -1;
            end else
            begin
              i := StrToInt(tmpStr);
              j := MainForm.S_BuyComboBox.Items.IndexOfObject(TObject(i));
              if j >= 0 then
               tmpStr := MainForm.S_BuyComboBox.Items[j];
            end;
            GoodsBuy.AddObject(tmpStr, TObject(i));
            ReadF(f, tmpStr);
          end;
          if NewFormat then
          begin
            ReadF(f, tmpStr);
            GoodsChange.Clear;
            while tmpStr <> '-1' do
            begin
              if tmpStr[Length(tmpStr)] = '~' then
              begin
                System.Delete(tmpStr, Length(tmpStr), 1);
                i := -1;
              end else
              begin
                i := StrToInt(tmpStr);
                j := MainForm.S_ChangeComboBox.Items.IndexOfObject(TObject(i));
                if j >= 0 then
                 tmpStr := MainForm.S_ChangeComboBox.Items[j];
              end;
              GoodsChange.AddObject(tmpStr, TObject(i));
              ReadF(f, tmpStr);
            end;
          end;
          ReadF(f, tmpStr); Msg1 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Msg2 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Msg3 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Msg4 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Msg5 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Msg6 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Msg7 := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr); Emotion := StrToInt(tmpStr);
          ReadF(f, tmpStr);
          if tmpStr = '0' then BitVector := ''
          else BitVector := tmpStr;
          ReadF(f, tmpStr); Seller := StrToInt(tmpStr);
          ReadF(f, tmpStr);
          if tmpStr = '0' then NotSellVector := ''
          else NotSellVector := tmpStr;
          ReadF(f, tmpStr);
          Rooms.Clear;
          while tmpStr <> '-1' do
          begin
            Rooms.Add(TObject(StrToInt(tmpStr)));
            ReadF(f, tmpStr);
          end;
          ReadF(f, tmpStr); Open1 := StrToInt(tmpStr);
          ReadF(f, tmpStr); Close1 := StrToInt(tmpStr);
          ReadF(f, tmpStr); Open2 := StrToInt(tmpStr);
          ReadF(f, tmpStr); Close2 := StrToInt(tmpStr);
        end;
        while (not Eof(f)) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
         ReadF(f, tmpStr);
      until Eof(f);
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
  except
//    Clear;
    if ShowErrMsg then
     MessageDlg('   '#13#10': ' + FileName +
                 FileErrMsg(FileCounter), mtError, [mbOk], 0);
  end;
end;

procedure TShopList.FillSource(Str: TStrings; ZoneNum: integer);
var
  i, j, Num: integer;
  s: string;
begin
  Str.Clear;
  Str.Add('CircleMUD v3.0 Shop File~'{'* BRusMUD shop file v1.0'});
  for i := 0 to 99 do
  begin
    Num := FindItem(ZoneNum*100 + i);
    if Num >= 0 then
     with TShop(Items[Num]) do
   begin
    Str.Add(Format('#%d E~', [Number]));
    for j := 0 to Goods.Count-1 do
     Str.Add(IntToStr(integer(Goods.Items[j])));
    Str.Add('-1');
    System.Str(CoeffSell:0:2, s); Str.Add(s);
    System.Str(CoeffBuy:0:2, s); Str.Add(s);
    System.Str(CoeffChange:0:2, s); Str.Add(s);
    for j := 0 to GoodsBuy.Count-1 do
     if integer(GoodsBuy.Objects[j]) = -1 then
      Str.Add(Format('%s~', [GoodsBuy[j]]))
     else
      Str.Add(Format('%d', [integer(GoodsBuy.Objects[j])]));
    Str.Add('-1');
    for j := 0 to GoodsChange.Count-1 do
     if integer(GoodsChange.Objects[j]) = -1 then
      Str.Add(Format('%s~', [GoodsChange[j]]))
     else
      Str.Add(Format('%d', [integer(GoodsChange.Objects[j])]));
    Str.Add('-1');
    Str.Add(Msg1 + '~');
    Str.Add(Msg2 + '~');
    Str.Add(Msg3 + '~');
    Str.Add(Msg4 + '~');
    Str.Add(Msg5 + '~');
    Str.Add(Msg6 + '~');
    Str.Add(Msg7 + '~');
    Str.Add(IntToStr(Emotion));
    if BitVector = '' then Str.Add('0')
    else Str.Add(BitVector);
    Str.Add(IntToStr(Seller));
    if NotSellVector = '' then Str.Add('0')
    else Str.Add(NotSellVector);
    for j := 0 to Rooms.Count-1 do
     Str.Add(IntToStr(integer(Rooms.Items[j])));
    Str.Add('-1');
    Str.Add(IntToStr(Open1));
    Str.Add(IntToStr(Close1));
    Str.Add(IntToStr(Open2));
    Str.Add(IntToStr(Close2));
   end;
  end;
  Str.Add('$'#13#10'$'#13#10'~');
end;

procedure TShopList.SaveToFile(FileName: string; ZoneNum: integer);
var
  Str: TStringList;
begin
  try
    Str := TStringList.Create;
    FillSource(Str, ZoneNum);
    Str.SaveToFile(FileName);
    Str.Free;
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TShopList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TShop(Items[i]).Free;
  inherited Destroy;
end;

{ TTrigger }

constructor TTrigger.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  Name := '';
  Model := 0;
  Flags := '';
  NumParam := 0;
  StrParam := '';
  Content := TStringList.Create;
end;

procedure TTrigger.SetToEditor;
begin
  with MainForm do
  begin
    T_NameEdit.Text := Self.Name;
    T_ClassComboBox.ItemIndex := T_ClassComboBox.Items.IndexOfObject(TObject(Model));
    T_ClassComboBoxChange(nil);
    case Model of
     0: SetCheckListBox(T_TypeMobCheckListBox, Flags);
     1: SetCheckListBox(T_TypeObjCheckListBox, Flags);
     2: SetCheckListBox(T_TypeRoomCheckListBox, Flags);
    end;
    T_NumArgEdit.Value := NumParam;
    T_ArgEdit.Text := StrParam;
    T_TextMemo.Lines.Clear; T_TextMemo.Lines.Add(RCL(Content.Text));
  end;
end;

procedure TTrigger.GetFromEditor;
begin
  with MainForm do
  begin
    Self.Name := T_NameEdit.Text;
    Model := integer(T_ClassComboBox.Items.Objects[T_ClassComboBox.ItemIndex]);
    case Model of
     0: GetCheckListBox(T_TypeMobCheckListBox, Flags);
     1: GetCheckListBox(T_TypeObjCheckListBox, Flags);
     2: GetCheckListBox(T_TypeRoomCheckListBox, Flags);
    end;
    NumParam := T_NumArgEdit.Value;
    StrParam := T_ArgEdit.Text;
    Content.Clear; Content.Add(RCL(T_TextMemo.Lines.Text));
  end;
end;

procedure TTrigger.Assign(Trigger: TTrigger);
begin
  Name := Trigger.Name;
  Model := Trigger.Model;
  Flags := Trigger.Flags;
  NumParam := Trigger.NumParam;
  StrParam := Trigger.StrParam;
  Content.Assign(Trigger.Content);
end;

destructor TTrigger.Destroy;
begin
  Content.Free;
  inherited Destroy;
end;

{ TTriggerList }

function TTriggerList.CheckNumber(_number: integer): boolean;
var
  i: integer;
  b: boolean;
begin
  b := True;
  for i := 0 to Count-1 do
   if TTrigger(Items[i]).Number = _number then
    b := False;
  Result := b;
end;

function TTriggerList.FindFreeNumber(ZoneNum: integer): byte;
var
  i: integer;
begin
  for i := 0 to 99 do
   if CheckNumber(ZoneNum*100 + i) then
    break;
  Result := i;
end;

function TTriggerList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TTrigger(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

procedure TTriggerList.Add(_number: integer);
var
  NewTrigger: TTrigger;
begin
  NewTrigger := TTrigger.Create(_number);
  inherited Add(NewTrigger);
end;

procedure TTriggerList.Refresh;
var
  i: integer;
begin
  TrigListForm.ListBox1.Clear;
  for i := 0 to Count-1 do
   with TTrigger(Items[i]) do
    TrigListForm.ListBox1.Items.AddObject(IntToStr(Number) + '  ' + Name, TObject(Number));
end;

function TTriggerList.NumToName(_number: integer): string;
var
  i: integer;
  s: string;
begin
  i := FindItem(_number);
  s := IntToStr(_number);
  if i >= 0 then s := s + ' ' + TTrigger(Items[i]).Name;
  Result := s;
end;

procedure TTriggerList.LoadFromFile(FileName: string);
var
  f: TextFile;
  tmpStr: string;
  ItemNumber: integer;
begin
  try
    FileCounter := -1;
    AssignFile(f, Filename); Reset(f);
    try
      FileCounter := 0;
      repeat
        ReadF(f, tmpStr);
      until Eof(f) or ((tmpStr <> '') and (tmpStr[1] = '#'));
      if not Eof(f) then
      repeat
        System.Delete(tmpStr, 1, 1);
        ItemNumber := FindItem(StrToInt(tmpStr));
        if ItemNumber < 0 then
        begin
          Add(StrToInt(tmpStr));
          ItemNumber := Count-1;
        end;
        with TTrigger(Items[ItemNumber]) do
        begin
          ReadF(f, tmpStr); Name := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr);
          Model := StrToInt(ExtractWord(1, tmpStr, [' ']));
          Flags := ExtractWord(2, tmpStr, [' ']);
          if Flags = '0' then Flags := ''
          else if (Length(Flags) = 1) or (Flags[2] > '9') then
           Flags := PrepareFlags(Flags);
          NumParam := StrToInt(ExtractWord(3, tmpStr, [' ']));
          ReadF(f, tmpStr); StrParam := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr);
          Content.Clear;
          while (tmpStr = '') or (tmpStr[Length(tmpStr)] <> '~') do
          begin
            Content.Add(tmpStr);
            ReadF(f, tmpStr);
          end;
          tmpStr := Copy(tmpStr, 1, Length(tmpStr)-1);
          if tmpStr <> '' then
           Content.Add(tmpStr);
        end;
        while (not Eof(f)) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
         ReadF(f, tmpStr);
      until Eof(f);
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
  except
//    Clear;
    if ShowErrMsg then
     MessageDlg('   '#13#10': ' + FileName +
                 FileErrMsg(FileCounter), mtError, [mbOk], 0);
  end;
end;

procedure TTriggerList.FillSource(Str: TStrings; ZoneNum: integer);
var
  i, Num: integer;
  s: string;
begin
  Str.Clear;
  Str.Add('* BRusMUD trigger file v1.0');
  for i := 0 to 99 do
  begin
    Num := FindItem(ZoneNum*100 + i);
    if Num >= 0 then
     with TTrigger(Items[Num]) do
   begin
    Str.Add(Format('#%d', [Number]));
    Str.Add(Name + '~');
    if Flags = '' then
     s := '0'
    else
     s := Flags;
    Str.Add(Format('%d %s %d', [Model, s, NumParam]));
    Str.Add(StrParam + '~');
    if Content.Count > 0 then
     Str.Add(Content.Text);
    Str.Add('~');
   end;
  end;
  Str.Add('$'#13#10'$');
end;

procedure TTriggerList.SaveToFile(FileName: string; ZoneNum: integer);
var
  Str: TStringList;
begin
  try
    Str := TStringList.Create;
    FillSource(Str, ZoneNum);
    Str.SaveToFile(FileName);
    Str.Free;
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TTriggerList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TTrigger(Items[i]).Free;
  inherited Destroy;
end;

{ TLinkObj }

constructor TLinkObj.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  IFFlag := False;
  Where := 0;
  Prob := 100;
end;

{ TLinkObjList }

procedure TLinkObjList.Assign(Source : TLinkObjList);
var
  i: integer;
  LinkObj: TLinkObj;
begin
  Clear;
  for i := 0 to Source.Count-1 do
  begin
    LinkObj := TLinkObj.Create(TLinkObj(Source.Items[i]).Number);
    LinkObj.IFFlag := TLinkObj(Source.Items[i]).IFFlag;
    LinkObj.Where := TLinkObj(Source.Items[i]).Where;
    LinkObj.Prob := TLinkObj(Source.Items[i]).Prob;
    Add(LinkObj);
  end;
end;

destructor TLinkObjList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TLinkObj(Items[i]).Free;
  inherited Destroy;
end;

{ TRoomObj }

constructor TRoomObj.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  IFFlag := False;
  Prob := 100;
  Objects := TLinkObjList.Create;
end;

procedure TRoomObj.Assign(RoomObj: TRoomObj);
begin
  Number := RoomObj.Number;
  IFFlag := RoomObj.IFFlag;
  Prob := RoomObj.Prob;
  Objects.Assign(RoomObj.Objects);
end;

destructor TRoomObj.Destroy;
begin
  Objects.Free;
  inherited Destroy;
end;

{ TRoomObjList }

function TRoomObjList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TRoomObj(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

destructor TRoomObjList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TRoomObj(Items[i]).Free;
  inherited Destroy;
end;

{ TRoomMob }

constructor TRoomMob.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  IFFlag := False;
  Follow := -1;
  Objects := TLinkObjList.Create;
end;

procedure TRoomMob.Assign(RoomMob: TRoomMob);
begin
  Number := RoomMob.Number;
  IFFlag := RoomMob.IFFlag;
  Follow := RoomMob.Follow;
  MaxQty := RoomMob.MaxQty;
  Objects.Assign(RoomMob.Objects);
end;

destructor TRoomMob.Destroy;
begin
  Objects.Free;
  inherited Destroy;
end;

{ TRoomMobList }

function TRoomMobList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TRoomMob(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

destructor TRoomMobList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TRoomMob(Items[i]).Free;
  inherited Destroy;
end;

{ TRoom }

constructor TRoom.Create(_number: integer);
var
  i: integer;
  DefaultExit: TExit;
begin
  inherited Create;
  Number := _number;
  x := 0; y := 0; z := 0;
  Name := '';
  Desc := TStringList.Create;
  Flags := '';
  Sector := 1;
  with DefaultExit do
  begin
    Name := '';
    Desc := '';
    Flag := 0;
    Key := -1;
    ExitRoom := 0;
    DefaultValue := 0;
  end;
  for i := 0 to 5 do
   Exits[i] := DefaultExit;
  Extra := TExtraDescList.Create;
  Triggers := TList.Create;
  Mobs := TRoomMobList.Create;
  Objects := TRoomObjList.Create;
  DelObjects := TList.Create;
end;

procedure TRoom.SetToEditor;
const
  DescTags: array [1..10] of string = (
   '<day>', '<night>', '<winterday>', '<winternight>', '<springday>', '<springnight>',
   '<summerday>', '<summernight>', '<autumnday>', '<autumnnight>'
  );
var
  i, j, p, DescNum: integer;
  s: string;
  RoomObj: TRoomObj;
  RoomMob: TRoomMob;
  R_FullDescArr: array [0..10] of TMemo;
begin
  with MainForm do
  begin
    R_DescEdit.Text := Self.Name;
    {}
    R_FullDescArr[0] := R_FullDescMemo;
    R_FullDescArr[1] := R_FullDescMemo1;
    R_FullDescArr[2] := R_FullDescMemo2;
    R_FullDescArr[3] := R_FullDescMemo3;
    R_FullDescArr[4] := R_FullDescMemo4;
    R_FullDescArr[5] := R_FullDescMemo5;
    R_FullDescArr[6] := R_FullDescMemo6;
    R_FullDescArr[7] := R_FullDescMemo7;
    R_FullDescArr[8] := R_FullDescMemo8;
    R_FullDescArr[9] := R_FullDescMemo9;
    R_FullDescArr[10] := R_FullDescMemo10;
    for i := 0 to 10 do
     R_FullDescArr[i].Lines.Clear;
    for i := 1 to MaxRoomAddDesc do RoomAddDesc[i] := '';
    i := 0;
    DescNum := 0;
    if Desc.Count > 0 then s := Desc[0];
    while i < Desc.Count do
     case DescNum of
     0: {normal}
     begin
       for j := 1 to 10 do
       begin
         p := Pos(DescTags[j], s);
         if p <> 0 then break;
       end;
       if p = 0 then
       begin
         R_FullDescMemo.Lines.Add(s);
         Inc(i);
         if i < Desc.Count then s := Desc[i];
       end else
       begin
         DescNum := j;
         if p <> 1 then R_FullDescMemo.Lines.Add(Copy(s, 1, p-1));
         System.Delete(s, 1, p + Length(DescTags[j]) - 1);
         if s = '' then
         begin
           Inc(i);
           if i < Desc.Count then s := Desc[i];
         end;
         if (Length(s) > 0) and (s[1] = 'R') then
         begin
           System.Delete(s, 1, 1);
           RoomAddDesc[DescNum] := 'R';
         end else
          RoomAddDesc[DescNum] := '';
       end;
     end;
     else {addition desc}
     begin
       p := Pos(DescTags[DescNum], s);
       if p = 0 then
       begin
         case DescNum of
          1: R_FullDescMemo1.Lines.Add(s);
          2: R_FullDescMemo2.Lines.Add(s);
          3: R_FullDescMemo3.Lines.Add(s);
          4: R_FullDescMemo4.Lines.Add(s);
          5: R_FullDescMemo5.Lines.Add(s);
          6: R_FullDescMemo6.Lines.Add(s);
          7: R_FullDescMemo7.Lines.Add(s);
          8: R_FullDescMemo8.Lines.Add(s);
          9: R_FullDescMemo9.Lines.Add(s);
          10: R_FullDescMemo10.Lines.Add(s);
         end;
         Inc(i);
         if i < Desc.Count then s := Desc[i];
       end else
       begin
         if p <> 1 then
          case DescNum of
           1: R_FullDescMemo1.Lines.Add(Copy(s, 1, p-1));
           2: R_FullDescMemo2.Lines.Add(Copy(s, 1, p-1));
           3: R_FullDescMemo3.Lines.Add(Copy(s, 1, p-1));
           4: R_FullDescMemo4.Lines.Add(Copy(s, 1, p-1));
           5: R_FullDescMemo5.Lines.Add(Copy(s, 1, p-1));
           6: R_FullDescMemo6.Lines.Add(Copy(s, 1, p-1));
           7: R_FullDescMemo7.Lines.Add(Copy(s, 1, p-1));
           8: R_FullDescMemo8.Lines.Add(Copy(s, 1, p-1));
           9: R_FullDescMemo9.Lines.Add(Copy(s, 1, p-1));
           10: R_FullDescMemo10.Lines.Add(Copy(s, 1, p-1));
          end;
         System.Delete(s, 1, p + Length(DescTags[DescNum]) - 1);
         if s = '' then
         begin
           Inc(i);
           if i < Desc.Count then s := Desc[i];
         end;
         DescNum := 0;
       end;
     end;
    end;
    for i := 0 to 10 do
     with R_FullDescArr[i] do
    begin
      SelStart := 0;
      SendMessage(Handle, EM_SCROLLCARET, 0, 0);
    end;
    if R_DescPageControl.ActivePage.TabIndex > 0 then
     R_DescCheckBox.Checked := RoomAddDesc[R_DescPageControl.ActivePage.TabIndex] <> '';
    SetCheckListBox(R_TypeCheckListBox, Flags);
    R_SectorComboBox.ItemIndex := R_SectorComboBox.Items.IndexOfObject(TObject(Sector));
    {}
    R_ExitNEdit.Value := Exits[0].ExitRoom;
    if Exits[0].ExitRoom = 0 then R_ExitNPanel.Visible := False
    else R_ExitNPanel.Visible := True;
    R_ExitEEdit.Value := Exits[1].ExitRoom;
    if Exits[1].ExitRoom = 0 then R_ExitEPanel.Visible := False
    else R_ExitEPanel.Visible := True;
    R_ExitSEdit.Value := Exits[2].ExitRoom;
    if Exits[2].ExitRoom = 0 then R_ExitSPanel.Visible := False
    else R_ExitSPanel.Visible := True;
    R_ExitWEdit.Value := Exits[3].ExitRoom;
    if Exits[3].ExitRoom = 0 then R_ExitWPanel.Visible := False
    else R_ExitWPanel.Visible := True;
    R_ExitUEdit.Value := Exits[4].ExitRoom;
    if Exits[4].ExitRoom = 0 then R_ExitUPanel.Visible := False
    else R_ExitUPanel.Visible := True;
    R_ExitDEdit.Value := Exits[5].ExitRoom;
    if Exits[5].ExitRoom = 0 then R_ExitDPanel.Visible := False
    else R_ExitDPanel.Visible := True;
    R_ExitNPanel.BevelOuter := bvRaised;
    R_ExitSPanel.BevelOuter := bvRaised;
    R_ExitEPanel.BevelOuter := bvRaised;
    R_ExitWPanel.BevelOuter := bvRaised;
    R_ExitUPanel.BevelOuter := bvRaised;
    R_ExitDPanel.BevelOuter := bvRaised;
    R_DoorEdit.Text := '';
    R_DoorDescEdit.Text := '';
    R_DoorComboBox.ItemIndex := -1;
    R_DoorKeyEdit.Value := -1;
    R_DoorDComboBox.ItemIndex := -1;
    CurrentDoor := -1;
    CurrentExits := Exits;
    { }
    R_AddDescListBox.Items.Clear;
    for i := 0 to Extra.Count-1 do
     R_AddDescListBox.Items.Add(TExtraDesc(Extra[i]).Alias);
    CurrentExtraDesc.Assign(Extra);
    if Extra.Count > 0 then
    begin
      R_AddDescListBox.ItemIndex := 0;
      R_AddDescListBoxClick(nil);
    end;
    {}
    R_TriggerListBox.Items.Clear;
    for i := 0 to Triggers.Count-1 do
     R_TriggerListBox.Items.AddObject(TriggerList.NumToName(integer(Triggers[i])), Triggers[i]);
    {}
    R_ObjectCheckListBox.Items.Clear;
    for i := 0 to Objects.Count-1 do
    begin
      RoomObj := TRoomObj.Create(TRoomObj(Objects[i]).Number);
      RoomObj.Assign(TRoomObj(Objects[i]));
      R_ObjectCheckListBox.Items.AddObject(ObjList.NumToName(RoomObj.Number), RoomObj);
      R_ObjectCheckListBox.Checked[i] := RoomObj.IFFlag;
    end;
    if Objects.Count > 0 then
     R_ObjectCheckListBox.ItemIndex := 0
    else
     R_ObjectCheckListBox.ItemIndex := -1;
    R_ObjectCheckListBoxClick(nil);
    {}
    R_MobCheckListBox.Items.Clear;
    for i := 0 to Mobs.Count-1 do
    begin
      RoomMob := TRoomMob.Create(TRoomMob(Mobs[i]).Number);
      RoomMob.Assign(TRoomMob(Mobs[i]));
      R_MobCheckListBox.Items.AddObject(MobList.NumToName(RoomMob.Number), RoomMob);
      R_MobCheckListBox.Checked[i] := RoomMob.IFFlag;
    end;
    if Mobs.Count > 0 then
     R_MobCheckListBox.ItemIndex := 0
    else
     R_MobCheckListBox.ItemIndex := -1;
    R_MobCheckListBoxClick(nil);
    {  }
    R_ODelCheckListBox.Items.Clear;
    for i := 0 to DelObjects.Count-1 do
    begin
      j := integer(DelObjects[i]) and $7FFFFFFF;
      R_ODelCheckListBox.Items.AddObject(ObjList.NumToName(j), DelObjects[i]);
      R_ODelCheckListBox.Checked[i] := (integer(DelObjects[i]) and $80000000) <> 0;
    end;
    { }
    R_SetExitPanel;
    CurrentExit := -1;
  end;
end;

procedure TRoom.GetFromEditor;
var
  i, j: integer;
  RoomObj: TRoomObj;
  RoomMob: TRoomMob;
begin
  with MainForm do
  begin
    Self.Name := R_DescEdit.Text;
    {description}
    Desc.Assign(R_FullDescMemo.Lines);
    if R_FullDescMemo1.Lines.Count > 0 then
     Desc.Add('<day>' + RoomAddDesc[1] + RCL(R_FullDescMemo1.Lines.Text) + '<day>');
    if R_FullDescMemo2.Lines.Count > 0 then
     Desc.Add('<night>' + RoomAddDesc[2] + RCL(R_FullDescMemo2.Lines.Text) + '<night>');
    if R_FullDescMemo3.Lines.Count > 0 then
     Desc.Add('<winterday>' + RoomAddDesc[3] + RCL(R_FullDescMemo3.Lines.Text) + '<winterday>');
    if R_FullDescMemo4.Lines.Count > 0 then
     Desc.Add('<winternight>' + RoomAddDesc[4] + RCL(R_FullDescMemo4.Lines.Text) + '<winternight>');
    if R_FullDescMemo5.Lines.Count > 0 then
     Desc.Add('<springday>' + RoomAddDesc[5] + RCL(R_FullDescMemo5.Lines.Text) + '<springday>');
    if R_FullDescMemo6.Lines.Count > 0 then
     Desc.Add('<springnight>' + RoomAddDesc[6] + RCL(R_FullDescMemo6.Lines.Text) + '<springnight>');
    if R_FullDescMemo7.Lines.Count > 0 then
     Desc.Add('<summerday>' + RoomAddDesc[7] + RCL(R_FullDescMemo7.Lines.Text) + '<summerday>');
    if R_FullDescMemo8.Lines.Count > 0 then
     Desc.Add('<summernight>' + RoomAddDesc[8] + RCL(R_FullDescMemo8.Lines.Text) + '<summernight>');
    if R_FullDescMemo9.Lines.Count > 0 then
     Desc.Add('<autumnday>' + RoomAddDesc[9] + RCL(R_FullDescMemo9.Lines.Text) + '<autumnday>');
    if R_FullDescMemo10.Lines.Count > 0 then
     Desc.Add('<autumnnight>' + RoomAddDesc[10] + RCL(R_FullDescMemo10.Lines.Text) + '<autumnnight>');

    GetCheckListBox(R_TypeCheckListBox, Flags);
    Sector := integer(R_SectorComboBox.Items.Objects[R_SectorComboBox.ItemIndex]);
    {}
    Exits := CurrentExits;
    { }
    Extra.Assign(CurrentExtraDesc);
    {}
    Triggers.Clear;
    for i := 0 to R_TriggerListBox.Items.Count-1 do
     Triggers.Add(R_TriggerListBox.Items.Objects[i]);
    {}
    Objects.Clear;
    for i := 0 to R_ObjectCheckListBox.Items.Count-1 do
    begin
      RoomObj := TRoomObj.Create(TRoomObj(R_ObjectCheckListBox.Items.Objects[i]).Number);
      RoomObj.Assign(TRoomObj(R_ObjectCheckListBox.Items.Objects[i]));
      RoomObj.IFFlag := R_ObjectCheckListBox.Checked[i];
      Objects.Add(RoomObj);
    end;
    {}
    Mobs.Clear;
    for i := 0 to R_MobCheckListBox.Items.Count-1 do
    begin
      RoomMob := TRoomMob.Create(TRoomMob(R_MobCheckListBox.Items.Objects[i]).Number);
      RoomMob.Assign(TRoomMob(R_MobCheckListBox.Items.Objects[i]));
      RoomMob.IFFlag := R_MobCheckListBox.Checked[i];
      Mobs.Add(RoomMob);
    end;
    {  }
    DelObjects.Clear;
    for i := 0 to R_ODelCheckListBox.Items.Count-1 do
    begin
      j := integer(R_ODelCheckListBox.Items.Objects[i]);
      if R_ODelCheckListBox.Checked[i] then
       j := j or $80000000;
      DelObjects.Add(TObject(j));
    end;
  end;
  {set in common list}
  i := RoomListForm.ListBox1.Items.IndexOfObject(TObject(Number));
  if i >= 0 then
   RoomListForm.ListBox1.Items[i] := RoomList.NumToName(Number)
  else
   RoomListForm.ListBox1.Items.AddObject(RoomList.NumToName(Number), TObject(Number));
end;

procedure TRoom.Assign(Room: TRoom);
var
  i: integer;
  RoomObj: TRoomObj;
  RoomMob: TRoomMob;
begin
//  x := Room.x; y := Room.y; z := Room.z;
  Name := Room.Name;
  Desc.Assign(Room.Desc);
  Flags := Room.Flags;
  Sector := Room.Sector;
//  Exits := Room.Exits;
  Extra.Assign(Room.Extra);
  Triggers.Clear;
  for i := 0 to Room.Triggers.Count-1 do
   Triggers.Add(TObject(integer(Room.Triggers.Items[i])));
  Mobs.Clear;
  for i := 0 to Room.Mobs.Count-1 do
   with TRoomMob(Room.Mobs.Items[i]) do
  begin
    RoomMob := TRoomMob.Create(Number);
    RoomMob.IFFlag := IFFlag;
    RoomMob.Follow := Follow;
    RoomMob.MaxQty := MaxQty;
    RoomMob.Objects.Assign(Objects);
    Mobs.Add(RoomMob);
  end;
  Objects.Clear;
  for i := 0 to Room.Objects.Count-1 do
   with TRoomObj(Room.Objects.Items[i]) do
  begin
    RoomObj := TRoomObj.Create(Number);
    RoomObj.IFFlag := IFFlag;
    RoomObj.Objects.Assign(Objects);
    Objects.Add(RoomObj);
  end;
  DelObjects.Clear;
  for i := 0 to Room.DelObjects.Count-1 do
   DelObjects.Add(TObject(integer(Room.DelObjects.Items[i])));
end;

destructor TRoom.Destroy;
begin
  Desc.Free;
  Extra.Free;
  Triggers.Free;
  Mobs.Free;
  Objects.Free;
  DelObjects.Free;
  inherited Destroy;
end;

{ TRoomList }

function TRoomList.CheckNumber(_number: integer): boolean;
var
  i: integer;
  b: boolean;
begin
  b := True;
  for i := 0 to Count-1 do
   if TRoom(Items[i]).Number = _number then
    b := False;
  Result := b;
end;

function TRoomList.FindFreeNumber(ZoneNum: integer): byte;
var
  i: integer;
begin
  for i := 0 to 99 do
   if CheckNumber(ZoneNum*100 + i) then
    break;
  Result := i;
end;

function TRoomList.FindItem(_number: integer): integer;
var
  i: integer;
  b: boolean;
begin
  b := False;
  for i := 0 to Count-1 do
   if TRoom(Items[i]).Number = _number then
  begin
    b := True;
    break;
  end;
  if b then Result := i
  else Result := -1;
end;

function TRoomList.NumToName(_number: integer): string;
var
  i: integer;
  s: string;
begin
  i := FindItem(_number);
  s := IntToStr(_number);
  if i >= 0 then s := s + ' ' + TRoom(Items[i]).Name;
  Result := s;
end;

procedure TRoomList.Add(_number: integer);
var
  NewRoom: TRoom;
begin
  NewRoom := TRoom.Create(_number);
  inherited Add(NewRoom);
end;

procedure TRoomList.Refresh;
var
  i: integer;
begin
  RoomListForm.ListBox1.Clear;
  for i := 0 to Count-1 do
   with TRoom(Items[i]) do
    RoomListForm.ListBox1.Items.AddObject(IntToStr(Number) + '  ' + Name, TObject(Number));
end;

procedure TRoomList.LoadFromFile(FileName: string);
var
  f: TextFile;
  tmpStr, tmpFN: string;
  i, j, ItemNumber: integer;
  ExtraDesc: TExtraDesc;
begin
  try
    tmpFN := '' + #13#10': ' + FileName;
    FileCounter := -1;
    AssignFile(f, Filename); Reset(f);
    try
      FileCounter := 0;
      repeat
        ReadF(f, tmpStr);
      until Eof(f) or ((tmpStr <> '') and (tmpStr[1] = '#'));
      if not Eof(f) then
      repeat
        System.Delete(tmpStr, 1, 1);
        ItemNumber := FindItem(StrToInt(tmpStr));
        if ItemNumber < 0 then
        begin
          Add(StrToInt(tmpStr));
          ItemNumber := Count-1;
        end;
        with TRoom(Items[ItemNumber]) do
        begin
          ReadF(f, tmpStr); Name := Copy(tmpStr, 1, Length(tmpStr)-1);
          ReadF(f, tmpStr);
          Desc.Clear;
          while (tmpStr = '') or (tmpStr[Length(tmpStr)] <> '~') do
          begin
            Desc.Add(tmpStr);
            ReadF(f, tmpStr);
          end;
          tmpStr := Copy(tmpStr, 1, Length(tmpStr)-1);
          if tmpStr <> '' then
           Desc.Add(tmpStr);
          ReadF(f, tmpStr);
          Flags := ExtractWord(2, tmpStr, [' ']);
          if Flags = '0' then Flags := '';
          Sector := StrToInt(ExtractWord(3, tmpStr, [' ']));
          ReadF(f, tmpStr);
          for i := 0 to 5 do Exits[i].ExitRoom := 0;
          while (tmpStr<>'') and (tmpStr[1] = 'D') do
           with Exits[Ord(tmpStr[2])-$30] do
          begin
            ReadF(f, tmpStr); Desc := Copy(tmpStr, 1, Length(tmpStr)-1);
            ReadF(f, tmpStr); Name := Copy(tmpStr, 1, Length(tmpStr)-1);
            ReadF(f, tmpStr);
            Flag := StrToInt(ExtractWord(1, tmpStr, [' ']));
            Key := StrToInt(ExtractWord(2, tmpStr, [' ']));
            ExitRoom := StrToInt(ExtractWord(3, tmpStr, [' ']));
            ReadF(f, tmpStr);
          end;
          Extra.Clear;
          while (tmpStr<>'') and (tmpStr[1] = 'E') do
          begin
            ExtraDesc := TExtraDesc.Create;
            ReadF(f, tmpStr);
            ExtraDesc.Alias := Copy(tmpStr, 1, Length(tmpStr)-1);
            ReadF(f, tmpStr);
            while (tmpStr = '') or (tmpStr[Length(tmpStr)] <> '~') do
            begin
              ExtraDesc.Desc.Add(tmpStr);
              ReadF(f, tmpStr);
            end;
            tmpStr := Copy(tmpStr, 1, Length(tmpStr)-1);
            if tmpStr <> '' then
             ExtraDesc.Desc.Add(tmpStr);
            Extra.Add(ExtraDesc);
            ReadF(f, tmpStr);
          end;
          while (tmpStr<>'S') do ReadF(f, tmpStr);
          ReadF(f, tmpStr);
          Triggers.Clear;
          while (tmpStr<>'') and (tmpStr[1] = 'T') do
          begin
            Triggers.Add(TObject(StrToInt(ExtractWord(2, tmpStr, [' ']))));
            ReadF(f, tmpStr);
          end;
          Mobs.Clear;
          Objects.Clear;
          DelObjects.Clear;
        end;
        while (not Eof(f)) and ((tmpStr = '') or (tmpStr[1] <> '#')) do
         ReadF(f, tmpStr);
      until Eof(f);
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
    {read map}
    tmpStr := ExtractFileName(FileName);
    i := Pos('.', tmpStr)-1; if i = 0 then i := Length(tmpStr);
    tmpStr := ExtractFilePath(FileName) + Copy(tmpStr, 1, i) + '.map';
    tmpFN := '' + #13#10': ' + tmpStr;
    FileCounter := -1;
    AssignFile(f, tmpStr); Reset(f);
    try
      FileCounter := 0;
      while not Eof(f) do
      begin
        ReadF(f, tmpStr);
        i := StrToInt(ExtractWord(1, tmpStr, [' ']));
        for j := 0 to Count-1 do
         with TRoom(Items[j]) do
          if i = Number then
        begin
          x := StrToInt(ExtractWord(2, tmpStr, [' ']));
          y := StrToInt(ExtractWord(3, tmpStr, [' ']));
          z := StrToInt(ExtractWord(4, tmpStr, [' ']));
          break;
        end;
      end;
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
  except
//    Clear;
    if ShowErrMsg then
     MessageDlg('   ' + tmpFN + FileErrMsg(FileCounter), mtError, [mbOk], 0);
  end;
end;

function ExtractPath_Name(s: string): string;
var
  st: string;
  i: integer;
begin
  st := ExtractFileName(s);
  i := Pos('.', st)-1; if i = 0 then i := Length(st);
  Result := ExtractFilePath(s) + Copy(st, 1, i);
end;

procedure TRoomList.FillSource(Str: TStrings; ZoneNum: integer);
var
  i, j, Num: integer;
  s: string;
begin
  Str.Clear;
  Str.Add('* BRusMUD room file v1.0');
  for i := 0 to 99 do
  begin
    Num := FindItem(ZoneNum*100 + i);
    if Num >= 0 then
     with TRoom(Items[Num]) do
   begin
    Str.Add(Format('#%d', [Number]));
    Str.Add(Name + '~');
    Str.Add(Desc.Text + '~');
    if Flags = '' then s := '0'
    else s := Flags;
    Str.Add(Format('%d %s %d', [Number div 100, s, Sector]));
    for j := 0 to 5 do
     with Exits[j] do
      if ExitRoom <> 0 then
    begin
      Str.Add(Format('D%d', [j]));
      Str.Add(Desc + '~');
      Str.Add(Name + '~');
      Str.Add(Format('%d %d %d', [Flag, Key, ExitRoom]));
    end;
    for j := 0 to Extra.Count-1 do
     with TExtraDesc(Extra.Items[j]) do
    begin
      Str.Add('E');
      Str.Add(Alias + '~');
      Str.Add(Desc.Text + '~');
    end;
    Str.Add('S');
    for j := 0 to Triggers.Count-1 do
     Str.Add(Format('T %d', [integer(Triggers.Items[j])]));
   end;
  end;
  Str.Add('$'#13#10'$');
end;

procedure TRoomList.SaveToFile(FileName: string; ZoneNum: integer);
var
  f: TextFile;
  i: integer;
  Str: TStringList;
begin
  try
    Str := TStringList.Create;
    FillSource(Str, ZoneNum);
    Str.SaveToFile(FileName);
    Str.Free;
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;

  try
    AssignFile(f, ExtractPath_Name(FileName) + '.map'); Rewrite(f);
    try
      for i := 0 to Count-1 do
       with TRoom(Items[i]) do
        if (Number div 100) = ZoneNum then
        {if z <> 0 then }WriteLn(f, Format('%d %d %d %d', [Number, x, y, z]));
    finally
      CloseFile(f);
    end;
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

procedure TRoomList.FillMap(ZoneNumber: integer);
var
  x, y, z: byte;
  i: integer;
begin
  { init zone }
  for z := MinZoneZ to MaxZoneZ do
   for x := MinZoneX to MaxZoneX do
    for y := MinZoneY to MaxZoneY do
     ZoneMap[x, y, z] := -1;
  { fill zone }
  MainForm.MapRoomsListBox.Clear;
  for i := 0 to Count-1 do
   with TRoom(Items[i]) do
   if (Number div 100) = ZoneNumber then
    if (z <> 0) then
     ZoneMap[x, y, z] := Number
    else
     MainForm.MapRoomsListBox.Items.AddObject(IntToStr(Number) + '  ' + Name, TObject(Number));
  { enable rooms edit }
{  with MainForm.R_IDComboBox do
   if Items.Count > 0 then
  begin
    CurrentRoom := RoomList.FindItem(integer(Items.Objects[ItemIndex]));
    TRoom(Items[CurrentRoom]).SetToEditor;
  end else
   CurrentRoom := -1;
  MainForm.MapDrawGrid.Refresh;}
end;

procedure TRoomList.DrawMiniMap(ZoneNum: integer);
var
  i: integer;
begin
  MainForm.MiniMapClear;
  for i := 0 to Count-1 do
   with TRoom(Items[i]) do
    if ((Number div 100) = ZoneNum) and (MainForm.CurrentLevel = z) then
     MainForm.MiniMapImage.Canvas.Pixels[x,y] := clBlack;
end;

destructor TRoomList.Destroy;
var
  i: integer;
begin
  for i := 0 to Count-1 do
   TRoom(Items[i]).Free;
  inherited Destroy;
end;

{ TZone }

constructor TZone.Create(_number: integer);
begin
  inherited Create;
  Number := _number;
  Name := '';
  RepopTime := 0;
  RepopType := 0;
  Objects := TRoomObjList.Create;
  DelMobs := TList.Create;
  FileName_ZON := '';
  FileName_WLD := '';
  FileName_OBJ := '';
  FileName_MOB := '';
  FileName_SHP := '';
  FileName_TRG := '';
end;

procedure TZone.Clear;
begin
  Number := 0;
  Name := '';
  RepopTime := 0;
  RepopType := 0;
  Objects.Clear;
  DelMobs.Clear;
end;

procedure TZone.SetToEditor;
var
  i, j: integer;
  RoomObj: TRoomObj;
begin
  with MainForm do
  begin
    Z_ZoneEdit.Value := Number;
    Z_NameEdit.Text := Self.Name;
    Z_RepopTimeEdit.Value := RepopTime;
    Z_RepopComboBox.ItemIndex := Z_RepopComboBox.Items.IndexOfObject(TObject(RepopType));
    {objects}
    Z_ObjCheckListBox.Items.Clear;
    for i := 0 to Objects.Count-1 do
    begin
      RoomObj := TRoomObj.Create(TRoomObj(Objects[i]).Number);
      RoomObj.Assign(TRoomObj(Objects[i]));
      Z_ObjCheckListBox.Items.AddObject(ObjList.NumToName(RoomObj.Number), RoomObj);
      Z_ObjCheckListBox.Checked[i] := RoomObj.IFFlag;
    end;
    { }
    Z_DelMobCheckListBox.Items.Clear;
    for i := 0 to DelMobs.Count-1 do
    begin
      j := integer(DelMobs.Items[i]) and $7FFFFFFF;
      Z_DelMobCheckListBox.Items.AddObject(MobList.NumToName(j), TObject(j));
      Z_DelMobCheckListBox.Checked[i] := (integer(DelMobs.Items[i]) and $80000000) <> 0;
    end;
    {file pathes}
    Z_ZoneFilenameEdit.Text := FileName_ZON;
    Z_RoomFilenameEdit.Text := FileName_WLD;
    Z_ObjFilenameEdit.Text := FileName_OBJ;
    Z_MobFilenameEdit.Text := FileName_MOB;
    Z_ShopFilenameEdit.Text := FileName_SHP;
    Z_TrigFilenameEdit.Text := FileName_TRG;
    {set objects}
    O_IDComboBox.Clear;
    with ObjList do
     for i := 0 to Count-1 do
      if (TObj(Items[i]).Number div 100) = Number then
       with TObj(Items[i]) do
        O_IDComboBox.Items.AddObject(IntToStr(Number) + '  ' + Imen, TObject(Number));
    if O_IDComboBox.Items.Count > 0 then
    begin
      O_IDComboBox.ItemIndex := 0;
      CurrentObj := ObjList.FindItem(integer(O_IDComboBox.Items.Objects[0]));
    end else
    begin
      O_IDComboBox.ItemIndex := -1;
      CurrentObj := -1;
    end;
    {set mobs}
    M_IDComboBox.Clear;
    with MobList do
     for i := 0 to Count-1 do
      if (TMob(Items[i]).Number div 100) = Number then
       with TMob(Items[i]) do
        M_IDComboBox.Items.AddObject(IntToStr(Number) + '  ' + Imen, TObject(Number));
    if M_IDComboBox.Items.Count > 0 then
    begin
      M_IDComboBox.ItemIndex := 0;
      CurrentMob := MobList.FindItem(integer(M_IDComboBox.Items.Objects[0]));
    end else
    begin
      M_IDComboBox.ItemIndex := -1;
      CurrentMob := -1;
    end;
    {set shops}
    S_IDComboBox.Clear;
    with ShopList do
     for i := 0 to Count-1 do
      if (TShop(Items[i]).Number div 100) = Number then
       with TShop(Items[i]) do
        S_IDComboBox.Items.AddObject(IntToStr(Number), TObject(Number));
    if S_IDComboBox.Items.Count > 0 then
    begin
      S_IDComboBox.ItemIndex := 0;
      CurrentShop := ShopList.FindItem(integer(S_IDComboBox.Items.Objects[0]));
    end else
    begin
      S_IDComboBox.ItemIndex := -1;
      CurrentShop := -1;
    end;
    {set triggers}
    T_IDComboBox.Clear;
    with TriggerList do
     for i := 0 to Count-1 do
      if (TTrigger(Items[i]).Number div 100) = Number then
       with TTrigger(Items[i]) do
        T_IDComboBox.Items.AddObject(IntToStr(Number) + '  ' + Name, TObject(Number));
    if T_IDComboBox.Items.Count > 0 then
    begin
      T_IDComboBox.ItemIndex := 0;
      CurrentTrigger := TriggerList.FindItem(integer(T_IDComboBox.Items.Objects[0]));
    end else
    begin
      T_IDComboBox.ItemIndex := -1;
      CurrentTrigger := -1;
    end;
    {set rooms}
    R_IDComboBox.Clear;
    with RoomList do
     for i := 0 to Count-1 do
      if (TRoom(Items[i]).Number div 100) = Number then
       with TRoom(Items[i]) do
        R_IDComboBox.Items.AddObject(IntToStr(Number) + '  ' + Name, TObject(Number));
    if R_IDComboBox.Items.Count > 0 then
    begin
      R_IDComboBox.ItemIndex := 0;
      CurrentRoom := RoomList.FindItem(integer(R_IDComboBox.Items.Objects[0]));
    end else
    begin
      R_IDComboBox.ItemIndex := -1;
      CurrentRoom := -1;
    end;
    RoomList.FillMap(Number);
    RoomList.DrawMiniMap(Number);
    {show zone statistics}
    ShowZoneStat;
    FindZoneErrors;
    FillSource(Z_SourceMemo.Lines);
  end;
end;

procedure TZone.GetFromEditor;
var
  i, j: integer;
  RoomObj: TRoomObj;
begin
  with MainForm do
  begin
    Number := Z_ZoneEdit.Value;
    Self.Name := Z_NameEdit.Text;
    RepopTime := Z_RepopTimeEdit.Value;
    RepopType := integer(Z_RepopComboBox.Items.Objects[Z_RepopComboBox.ItemIndex]);
    {objects}
    Objects.Clear;
    for i := 0 to Z_ObjCheckListBox.Items.Count-1 do
    begin
      RoomObj := TRoomObj.Create(0);
      RoomObj.Assign(TRoomObj(Z_ObjCheckListBox.Items.Objects[i]));
      RoomObj.IFFlag := Z_ObjCheckListBox.Checked[i];
      Objects.Add(RoomObj);
    end;
    { }
    DelMobs.Clear;
    with Z_DelMobCheckListBox do
     for i := 0 to Items.Count-1 do
    begin
      j := integer(Items.Objects[i]);
      if Checked[i] then
       j := j or $80000000;
      DelMobs.Add(TObject(j));
    end;
    {filepathes}
    FileName_ZON := Z_ZoneFilenameEdit.Text;
    FileName_WLD := Z_RoomFilenameEdit.Text;
    FileName_OBJ := Z_ObjFilenameEdit.Text;
    FileName_MOB := Z_MobFilenameEdit.Text;
    FileName_SHP := Z_ShopFilenameEdit.Text;
    FileName_TRG := Z_TrigFilenameEdit.Text;
  end;
end;

function TZone.LoadZone(FileName: string): boolean;
var
  Path: string;
begin
  ShowErrMsg := False;
  Result := LoadFromFile(FileName);
  FileName_ZON := FileName;
  Path := ExtractFilePath(FileName); Path := Copy(Path, 1, Length(Path)-4);
  FileName_WLD := Path + 'WLD\' + IntToStr(Number) + '.wld';
  FileName_OBJ := Path + 'OBJ\' + IntToStr(Number) + '.obj';
  FileName_MOB := Path + 'MOB\' + IntToStr(Number) + '.mob';
  FileName_SHP := Path + 'SHP\' + IntToStr(Number) + '.shp';
  FileName_TRG := Path + 'TRG\' + IntToStr(Number) + '.trg';
  ShowErrMsg := True;
end;

function TZone.LoadFromFile(FileName: string): boolean;
var
  f: TextFile;
  tmpStr: string;
  i, Num, Room: integer;
  CurrObj, CurrMob, CurrRoom: integer;
  RoomObj: TRoomObj;
  RoomMob: TRoomMob;
  LinkObj: TLinkObj;

procedure RemoveBeginSpaces(var s: string);
var
  i: integer;
begin
  i := 1;
  while (i < Length(s)) and  (s[i] in [' ', #9]) do Inc(i);
  if i > 1 then Delete(s, 1, i-1);
end;

begin
  try
    FileCounter := -1;
    AssignFile(f, Filename); Reset(f);
    try
      Clear;
      FileCounter := 0;
      repeat
        ReadF(f, tmpStr);
      until Eof(f) or ((tmpStr <> '') and (tmpStr[1] = '#'));
      Val(Copy(tmpStr, 2, Length(tmpStr)-1), Number, i);
      ReadF(f, tmpStr);
      Name := Copy(tmpStr, 1, Length(tmpStr)-1);
      ReadF(f, tmpStr);
      RepopTime := StrToInt(ExtractWord(2, tmpStr, [' ']));
      RepopType := StrToInt(ExtractWord(3, tmpStr, [' ']));
      ReadF(f, tmpStr); RemoveBeginSpaces(tmpStr);
      Objects.Clear;
      DelMobs.Clear;
      CurrObj := -1; CurrMob := -1; CurrRoom := -1;
      while (not Eof(f)) and (tmpStr <> 'S') do
      begin
        case UpCase(tmpStr[1]) of
         'Q': {Q ifflag mob_vnum -    }
         begin
           i := (StrToInt(ExtractWord(2, tmpStr, [' '])) shl 31) or
            (StrToInt(ExtractWord(3, tmpStr, [' '])));
           DelMobs.Add(TObject(i));
         end;
         'O': {O ifflag obj_vnum max_in_world room_vnum obj_prob -  }
         begin
           Room := StrToInt(ExtractWord(5, tmpStr, [' ']));
           i := StrToInt(ExtractWord(2, tmpStr, [' '])) shl 31;
           Num := StrToInt(ExtractWord(3, tmpStr, [' ']));
           RoomObj := TRoomObj.Create(Num);
           RoomObj.IFFlag := i <> 0;
           RoomObj.Prob := StrToInt(ExtractWord(6, tmpStr, [' ', #9]));
           if RoomObj.Prob = -1 then RoomObj.Prob := 100;
           if Room < 0 then
            Objects.Add(RoomObj)
           else
           begin
             CurrRoom := RoomList.FindItem(Room);
             if CurrRoom >= 0 then
              TRoom(RoomList.Items[CurrRoom]).Objects.Add(RoomObj);
           end;
           CurrObj := ObjList.FindItem(Num);
           if CurrObj >= 0 then
            TObj(ObjList.Items[CurrObj]).MaxInWorld := StrToInt(ExtractWord(4, tmpStr, [' ', #9]));
         end;
         'M': {M ifflag mob_vnum max_in_world room_vnum max_in_room -    }
         begin
           Room := StrToInt(ExtractWord(5, tmpStr, [' ']));
           Num := StrToInt(ExtractWord(3, tmpStr, [' ']));
           CurrRoom := RoomList.FindItem(Room);
           if CurrRoom >= 0 then
            with TRoom(RoomList.Items[CurrRoom]) do
           begin
             RoomMob := TRoomMob.Create(Num);
             RoomMob.IFFlag := (StrToInt(ExtractWord(2, tmpStr, [' '])) shl 31) <> 0;
             RoomMob.MaxQty := StrToInt(ExtractWord(6, tmpStr, [' ', #9]));
             Mobs.Add(RoomMob);
             CurrMob := MobList.FindItem(Num);
             if CurrMob >= 0 then
              TMob(MobList.Items[CurrMob]).MaxInWorld := StrToInt(ExtractWord(4, tmpStr, [' ']));
           end;
         end;
         'F': {F ifflag room_vnum mob_vnum_leader mob_vnum_follower -    }
         begin
           Room := StrToInt(ExtractWord(3, tmpStr, [' ']));
           Num := StrToInt(ExtractWord(5, tmpStr, [' ']));
           i := RoomList.FindItem(Room);
           if i >= 0 then
            with TRoom(RoomList.Items[i]).Mobs do
           begin
             Num := FindItem(Num);
             if Num >= 0 then
              TRoomMob(Items[Num]).Follow := StrToInt(ExtractWord(4, tmpStr, [' ']));
           end;
         end;
         'P': {P ifflag obj_vnum max_in_world objto_vnum obj_prob -    }
         begin
           Num := StrToInt(ExtractWord(3, tmpStr, [' ']));
           if CurrRoom >= 0 then
            with TRoom(RoomList.Items[CurrRoom]) do
             if Objects.Count > 0 then
              with TRoomObj(Objects.Items[Objects.Count-1]) do
           begin
             LinkObj := TLinkObj.Create(Num);
             LinkObj.IFFlag := ExtractWord(2, tmpStr, [' ']) <> '0';
             LinkObj.Prob := StrToInt(ExtractWord(6, tmpStr, [' ', #9]));
             if LinkObj.Prob = -1 then LinkObj.Prob := 100;
             Objects.Add(LinkObj);
             CurrObj := ObjList.FindItem(Num);
             if CurrObj >= 0 then
              TObj(ObjList.Items[CurrObj]).MaxInWorld := StrToInt(ExtractWord(4, tmpStr, [' ', #9]));
           end;
         end;
         'G', {G ifflag obj_vnum max_in_world ??? obj_prob -   }
         'E': {E ifflag obj_vnum max_in_world obj_pos obj_prob -  }
         begin
           Num := StrToInt(ExtractWord(3, tmpStr, [' ']));
           if CurrRoom >= 0 then
            with TRoom(RoomList.Items[CurrRoom]) do
             if Mobs.Count > 0 then
              with TRoomMob(Mobs.Items[Mobs.Count-1]) do
           begin
             LinkObj := TLinkObj.Create(Num);
             LinkObj.IFFlag := ExtractWord(2, tmpStr, [' ']) <> '0';
             if UpCase(tmpStr[1]) = 'G' then
             begin
               LinkObj.Where := -1;
               LinkObj.Prob := StrToInt(ExtractWord(5, tmpStr, [' ', #9]));
             end else
             begin
               LinkObj.Where := StrToInt(ExtractWord(5, tmpStr, [' ', #9]));
               LinkObj.Prob := StrToInt(ExtractWord(6, tmpStr, [' ', #9]));
             end;
             if LinkObj.Prob = -1 then LinkObj.Prob := 100;
             Objects.Add(LinkObj);
             CurrObj := ObjList.FindItem(Num);
             if CurrObj >= 0 then
              TObj(ObjList.Items[CurrObj]).MaxInWorld := StrToInt(ExtractWord(4, tmpStr, [' ', #9]));
           end;
         end;
         'R': {R ifflag room_vnum obj_vnum -    }
         begin
           Room := RoomList.FindItem(StrToInt(ExtractWord(3, tmpStr, [' '])));
           if Room >= 0 then
           begin
             i := StrToInt(ExtractWord(2, tmpStr, [' '])) shl 31;
             Num := StrToInt(ExtractWord(4, tmpStr, [' '])) or i;
             TRoom(RoomList.Items[Room]).DelObjects.Add(TObject(Num));
           end;
         end;
         'D': {D ifflag room_vnum dir(0-5) door_flag -    }
         begin
           i := RoomList.FindItem(StrToInt(ExtractWord(3, tmpStr, [' '])));
           if i >= 0 then
            with TRoom(RoomList.Items[i]).Exits[StrToInt(ExtractWord(4, tmpStr, [' ']))] do
             DefaultValue := StrToInt(ExtractWord(5, tmpStr, [' ', #9]));
         end;
         'T': {T ifflag trgflag trgvnum -      }
         begin
           i := StrToInt(ExtractWord(3, tmpStr, [' ']));
           Num := (StrToInt(ExtractWord(2, tmpStr, [' '])) shl 31) or
                  (StrToInt(ExtractWord(4, tmpStr, [' '])));
           case i of
            0: {mob}
             if CurrMob >= 0 then
              TMob(MobList.Items[CurrMob]).Triggers.Add(TObject(Num));
            1: {object}
             if CurrObj >= 0 then
              TObj(ObjList.Items[CurrObj]).Triggers.Add(TObject(Num));
            2: {room}
             if CurrRoom >= 0 then
              TRoom(RoomList.Items[CurrRoom]).Triggers.Add(TObject(Num));
           end; {case}
         end;
         // V ifflag trgflag context room_vnum varname value -     
        end;
        ReadF(f, tmpStr); RemoveBeginSpaces(tmpStr);
      end;
      FileCounter := -2;
    finally
      CloseFile(f);
    end;
    Result := True;
  except
    if ShowErrMsg then
     MessageDlg('   '#13#10': ' + FileName +
                 FileErrMsg(FileCounter), mtError, [mbOk], 0);
    Result := False;
  end;
end;

procedure TZone.FillSource(Str: TStrings);
var
  i, j, k, a: integer;
  RoomNum: integer;
  s: string;
begin
  Str.Clear;
  Str.Add('* BRusMUD zone file v1.0');
  Str.Add(Format('#%d', [Number]));
  Str.Add(Name + '~');
  Str.Add(Format('%d %d %d', [Number*100+99, RepopTime, RepopType]));
  {O}
  for i := 0 to Objects.Count-1 do
   with TRoomObj(Objects[i]) do
  begin
    k := ObjList.FindItem(Number);
    s := '';
    if k >= 0 then
    begin
      s := #9'(' + TObj(ObjList.Items[k]).Imen + ')';
      k := TObj(ObjList.Items[k]).MaxInWorld;
    end;
    Str.Add(Format('O %d %d %d -1 %d%s', [Ord(IFFlag), Number, k, Prob, s]));
  end;
  {Q}
  for i := 0 to DelMobs.Count-1 do
   Str.Add(Format('Q %d %d -1 -1 -1', [integer(DelMobs.Items[i]) shr 31,
           integer(DelMobs.Items[i]) and $7FFFFFFF]));
  {rooms}
  for i := 0 to RoomList.Count-1 do
   if (TRoom(RoomList.Items[i]).Number div 100) = Number then
    with TRoom(RoomList.Items[i]) do
  begin
    RoomNum := Number;
    {R}
    for j := 0 to DelObjects.Count-1 do
    begin
      k := ObjList.FindItem(integer(DelObjects[j]) and $7FFFFFFF);
      if k >= 0 then
       s := #9'(' + TObj(ObjList.Items[k]).Imen + ')'
      else
       s := '';
      Str.Add(Format('R %d %d %d -1 -1%s', [integer(DelObjects[j]) shr 31,
       RoomNum, integer(DelObjects[j]) and $7FFFFFFF, s]));
    end;
    {O}
    for j := 0 to Objects.Count-1 do
     with TRoomObj(Objects[j]) do
    begin
      k := ObjList.FindItem(Number);
      s := '';
      if k >= 0 then
      begin
        s := #9'(' + TObj(ObjList.Items[k]).Imen + ')';
        k := TObj(ObjList.Items[k]).MaxInWorld;
      end;
      Str.Add(Format('O %d %d %d %d %d%s', [Ord(IFFlag), Number, k, RoomNum, Prob, s]));
      {P}
      for k := 0 to Objects.Count-1 do
      begin
        a := ObjList.FindItem(TLinkObj(Objects[k]).Number);
        s := '';
        if a >= 0 then
        begin
          s := #9'(' + TObj(ObjList.Items[a]).Imen + ')';
          a := TObj(ObjList.Items[a]).MaxInWorld;
        end;
        Str.Add(Format('P %d %d %d %d %d%s', [Ord(TLinkObj(Objects[k]).IFFlag),
         TLinkObj(Objects[k]).Number, a, Number, TLinkObj(Objects[k]).Prob, s]));
      end;
    end;
    {M}
    for j := 0 to Mobs.Count-1 do
     with TRoomMob(Mobs[j]) do
    begin
      k := MobList.FindItem(Number);
      s := '';
      if k >= 0 then
      begin
        s := #9'(' + TMob(MobList.Items[k]).Imen + ')';
        k := TMob(MobList.Items[k]).MaxInWorld;
      end;
      Str.Add(Format('M %d %d %d %d %d%s', [Ord(IFFlag), Number, k, RoomNum, MaxQty, s]));
      {G, E}
      for k := 0 to Objects.Count-1 do
      begin
        a := ObjList.FindItem(TLinkObj(Objects[k]).Number);
        s := '';
        if a >= 0 then
        begin
          s := #9'(' + TObj(ObjList.Items[a]).Imen + ')';
          a := TObj(ObjList.Items[a]).MaxInWorld;
        end;
        if TLinkObj(Objects[k]).Where = -1 then
         Str.Add(Format('G %d %d %d %d -1%s', [Ord(TLinkObj(Objects[k]).IFFlag),
          TLinkObj(Objects[k]).Number, a, TLinkObj(Objects[k]).Prob, s]))
        else
         Str.Add(Format('E %d %d %d %d %d%s', [Ord(TLinkObj(Objects[k]).IFFlag),
          TLinkObj(Objects[k]).Number, a, TLinkObj(Objects[k]).Where,
          TLinkObj(Objects[k]).Prob, s]));
      end;
      {F}
      if Follow <> -1 then
      begin
        k := MobList.FindItem(Follow);
        if k >= 0 then
         s := #9'(' + TMob(MobList.Items[k]).Imen + ')'
        else
         s := '';
        Str.Add(Format('F 0 %d %d %d -1%s', [RoomNum, Follow, Number, s]));
      end;
    end;
    {D}
    for j := 0 to 5 do
     if (Exits[j].ExitRoom <> 0) and (Exits[j].Flag > 0) then
      Str.Add(Format('D 0 %d %d %d -1'#9'(%s)', [RoomNum, j, Exits[j].DefaultValue, Name]));
  end;
  Str.Add('S'#13#10'$'#13#10'$');
end;

procedure TZone.SaveToFile(FileName: string);
begin
  try
    FillSource(MainForm.Z_SourceMemo.Lines);
    MainForm.Z_SourceMemo.Lines.SaveToFile(FileName);
  except
    if ShowErrMsg then
     MessageDlg('   ', mtError, [mbOk], 0);
  end;
end;

destructor TZone.Destroy;
begin
  Objects.Clear;
  DelMobs.Clear;
  inherited Destroy;
end;

end.
